#pragma rtGlobals=3		// Use modern global access method and strict wave access.
#pragma version = 1.21
#pragma ModuleName=DHarmonic
#requiredPackages "Herix;VoigtDeconvolve;"
#initFunctionName "Init_DampedHarmonic()"

// NOTE, T_K is an OVERRIDE variable, regardless of what is in the spec file, T_K is used.
//		to use the spec value for Temperature, set T_K to NaN
Constant T_K_default = 300			// default temperature (K), when all else fails, use this.
StrConstant Temperature_PVorWave = "Heat_Stage_T2"
Constant excludeHWdefault = 8		// ignore the 8 points about zero meV
Constant excludeHWdefaultE = 3
Constant kB = 0.086173303				// Boltzman constant (meV/K)
Constant NphononPeaks_MAX = 11		// maximum number of phonons peaks (11 means 5 phonons + 1 center)

//	Definition of chi^2
//
//		data[i]		measured y data values
//		x[i]			x values for each y data[i]
//		calc(x)		calculation (based on coefs)
//		calc(x[i])	calculated value for each data point
//		sigma[i]		sigma for each data[i], assuming sigma[i] = sqrt(data[i])
//
//	chi^2 = Sum{ [ (data[i] - calc(x[i]) / sigma[i] ]^2 }



Menu "Herix"
	SubMenu "Damped-Harmonic"
		DHarmonic#MenuItemIfPhononPeakInofOnGraph("Auto Find & Fit Phonon Peaks",not=1), AutoFindPhononPeaks("",$"",$"",$"")
//		"Add Phonon peak to Graph for fitting","", AddPhononPeakMarker("")
		SubMenu "Old Methods"
			SubMenu "Single-Scan [point based]"
				MenuItemIfTopGraphHerixEscan("Fit Dampled Harmonic to Top Graph"), FitDampedHarmonic($"",$"")
				"Set Damped Harmonic Coefs, Multi", SetDampedHarmonicCoefs($"",$"",Nph=NaN)
				"  Set Central Exclusion HW points...", SetCentralExclusionHWp(NaN)
			End
			SubMenu "Multiple-Scan [meV based]"
				MenuItemIfTopGraphHerixEscan("Fit Dampled Harmonic to Top Graph"), FitDampedHarmonicMultiple()
				"  Set Central Exclusion HW meV...", SetCentralExclusionHWE(NaN)
			End
		End
		MenuItemIfWaveClassExists("  Table of Coefficients","DampedHarmonicCoefs","DIMS:1,MINROWS:4"), CoefsTable($"")
		MenuItemIfWaveClassExists("  Table of Coefficients","DampedHarmonicCoefs","DIMS:1,MINROWS:4"), DisplayTableOfWave($"",classes="DampedHarmonicCoefs",promptStr="Coefficients",options="DIMS:1,MINROWS:4")
	End
	"  OverRide Sample Temperature...", OverRideSampleTemperature(-1)			// Negative forces a dialog (because NaN is valid)
	"  Set PV or Wave Name for Temperature...", SetTemperaturePVorWaveName("")
End

// These following 3 menus are for the multi-phonon fitting
Menu "TracePopup"
	"-"
	DHarmonic#MenuItemIfPhononPeakInofOnGraph("Auto Find & Fit Phonon Peaks",not=1), AutoFindPhononPeaks("",$"",$"",$"")
	DHarmonic#MenuItemIfPhononPeakInofOnGraph("Fit Identified Phonon Peaks"), FitDHarmonicMultipleFromMarkers()
	DHarmonic#MenuItemIfPhononPeakInofOnGraph("Clear All Phonon Peak Markers"), ClearAllPhononPeakMarkers("")
	DHarmonic#MenuItemIfPhononPeakInofOnGraph("Update Phonon Markers"), DHarmonic#UpdatePhononMarkersFromUserData("")
End
Menu "AllTracesPopup"
	"-"
	DHarmonic#MenuItemIfPhononPeakInofOnGraph("Auto Find & Fit Phonon Peaks",not=1), AutoFindPhononPeaks("",$"",$"",$"")
	DHarmonic#MenuItemIfPhononPeakInofOnGraph("Fit Identified Phonon Peaks"), FitDHarmonicMultipleFromMarkers()
	DHarmonic#MenuItemIfPhononPeakInofOnGraph("Clear All Phonon Peak Markers"), ClearAllPhononPeakMarkers("")
End

Menu "GraphMarquee"
	"-"
	DHarmonic#MenuItemIfNoResolution("Make A Resolution Wave...","peakWave;Resolution*","DIMS:1"), /Q, herix#SetResolutionWave($"")
	MenuItemsWaveClassOnGraph("Add Phonon peak to Graph for fitting","peakWave;Resolution*;specWave*",""), /Q, AddPhononPeakMarker("")
	DHarmonic#MenuItemIfPhononPeakInofOnGraph("Remove 1 phonon Peak Marker"), /Q, RemovePhononPeakMarker("")
	DHarmonic#MenuItemIfPhononPeakInofOnGraph("Auto Find & Fit Phonon Peaks",not=1), AutoFindPhononPeaks("",$"",$"",$"")
	DHarmonic#MenuItemIfPhononPeakInofOnGraph("Fit Identified Phonon Peaks"), FitDHarmonicMultipleFromMarkers()
	DHarmonic#MenuItemIfPhononPeakInofOnGraph("Clear All Phonon Peak Markers"), ClearAllPhononPeakMarkers("")
End


Static Function/S MenuItemIfPhononPeakInofOnGraph(item,[not])
	String item						// string that you want to appear in menu
	Variable not					// menu appears if info NOT on graph
	not = ParamIsDefault(not) || numtype(not) ? 0 : !(!not)
	String gName="", uList=""
	if (ItemsInList(WinList("*",";","WIN:1,VISIBLE:1")))	// a graph is present
		uList = GetUserData(gName,"","PhononPeaks")	// get ulist from top graph
	endif
	Variable show = strlen(uList)>0
	show = not ? !show : show
	if (show)														// if something in ulist, enable menu
		return item
	endif
	return "("+item												// ulist empty, disable menu
End


Static Function/S MenuItemIfNoResolution(item,classes,optionsStr)
	// returns "" if a resolution wave exists, if no resolutionWave then it provides the menu selection
	String item						// string that you want to appear in menu
	String classes					// semi-colon separated list of wave classes, can use "*" in classes
	String optionsStr				// options that are found in WaveList function optionsStr
	Variable invisible			// controls menu item when conditions not met: true -> menu item absent, false or absent -> menu item grayed out
	if (strlen(WaveListClass(classes,"*",optionsStr))>0)
		return ""
	endif
	if (strlen(WaveListClass(classes,"*",optionsStr,fldr="root:"))>0)
		return ""
	endif
	return item
End




//  ======================================================================================  //
//  ======================= Start of Multi-Phonon Interactive Fit ========================  //
//
//  This uses Marquees to let the user select the phonon (or central peak) locations
//		This information is stored in the graph, and it then fits the spectra using that information

Function AutoFindPhononPeaks(gName,wy,wx,wyErr)		// automaticaly try to find the phonon peaks, and optionallty fit too
	String gName
	Wave wy, wx, wyErr

	if (!WaveExists(wy))
		Wave/WAVE yxWaves = TotalWavesToFitFromGraph(gName)
		Wave wy = yxWaves[0]
		Wave wx = yxWaves[1]
		Wave wyErr = yxWaves[2]
		WaveClear yxWaves
		if (strlen(gName)==0)
			String str = StringByKey("sourceY",note(wy),"=")
			str = StringFromList(0,str,",")
			Wave ww = $str
			gName = StringFromList(0,WindowsWithWave(ww,1))
		endif
//		String list = SpecDataAndXrangeOnGraph("")
//		String traceName = StringFromList(0,StringFromList(0,list),",")
//		Wave wy = TraceNameToWaveRef(gName,traceName)
//		Wave wx = XWaveRefFromTrace(gName,traceName)
//		Wave wyErr = GetErrorWave(wy)						// get error wave if it exists
//		if (strlen(gName)<1)
//			gName = StringFromList(0,WindowsWithWave(wy,1))
//		endif
	endif
	if (!WaveExists(wy) || strlen(gName)<1)
		return 1
	endif

	String PhononPeaksFree=MakePhononPeaksUserData(wy,wx,wyErr), PhononPeaks=""

	// reform PhononPeaksFree string
	String item,pkStr, trName, trList=TraceNamesInClass("peakWave;Resolution*;specWave*",gName)
	Variable i, xlo,xhi, m,Ntraces=ItemsInList(trList), ilo,ihi,Nwx, Ethresh=-1e-4
	for (i=0;i<ItemsInList(PhononPeaksFree);i+=1)
		item = StringFromList(i,PhononPeaksFree)
		pkStr = StringFromList(0,item,":")
		xlo = str2num(StringFromList(3,item,","))
		xhi = str2num(StringFromList(4,item,","))
		trName = ""
		for (m=0;m<Ntraces;m+=1)
			Wave wxi = XWaveRefFromTrace(gName,StringFromList(m,trList))
			Nwx = DimSize(wxi,0)
			if ( ((xlo-wxi[0])>Ethresh) && ((wxi[Nwx-1]-xhi)>Ethresh) & (Nwx>1))		// This trace contains [xlo,xhi]
				trName = StringFromList(m,trList)
				ilo = BinarySearch(wxi,xlo)
				ihi = BinarySearch(wxi,xhi)
				ilo = ilo==-1 ? 0 : ilo
				ihi = ihi==-2 ? Nwx-1 : ihi
				xlo = wxi[ilo]
				xhi = wxi[ihi]
				sprintf str, "%s:%s,%d,%d,%g,%g;", pkStr,trName,ilo,ihi,xlo,xhi
				PhononPeaks += str
				break
			endif
		endfor
	endfor
	if (strlen(PhononPeaks)<1)
		DoAlert 0, "UNable to Auto Find Phonon Peaks."
		return 1
	endif

	ClearAllPhononPeakMarkers(gName)
	SetWindow $gName userdata(PhononPeaks)=PhononPeaks
	UpdatePhononMarkersFromUserData(gName)
	DoUpdate/W=$gName

	DoAlert 1, "Fit the Phono Peaks?"
	if (V_flag==1)
		FitDHarmonicMultipleFromMarkers()
	endif
End
//
// use this to replace code in FitDHarmonicMultipleFromMarkers() and FitDampedHarmonicMultiple()
Static Function/WAVE TotalWavesToFitFromGraph(win)
	// produce a combined wave containing all the data on a particular graph
	String win				// graph window, use "" for top graph

	String traceList=TraceNamesInClass("Resolution*;specWave*",win)
	String traceNameStr, usedWavesNames="", wnote="", yList="", xList=""
	Make/N=0/D/FREE yTotal, xTotal, yTotalErr=NaN	// always a point at 0
	Variable i, Ni, Ntotal, Nwaves=ItemsInList(traceList)
	Variable T_K = NumVarOrDefault("root:Packages:DampedHarmonic:T_K",NaN)	// this is an OVERRIDE value
	for (i=0,Ntotal=0; i<Nwaves; i+=1)
		traceNameStr = StringFromList(i,traceList)
		Wave ywi = TraceNameToWaveRef(win,traceNameStr)
		Wave xwi = XWaveRefFromTrace(win, traceNameStr )
		if (!WaveExists(ywi))
			continue												// does not exist
		elseif (WhichListItem(GetWavesDataFolder(ywi,2),usedWavesNames)>=0)
			continue												// already included
		endif
		Ni = numpnts(ywi)
		if (Ni<1)
			continue												// empty
		endif
		if (strlen(wnote)==0)
			wnote = note(ywi)
			T_K  = T_K>0 ? T_K : NumberByKey("T_Sample",wnote,"-")
		endif
		usedWavesNames += GetWavesDataFolder(ywi,2)+";"
		T_K = T_K>0 ? T_K : T_SampleFromYwave(ywi,NaN)
		yList += fwhm#prettyWaveName(ywi)+","
		xList += fwhm#prettyWaveName(xwi)+","
		Redimension/N=(Ntotal+Ni) yTotal, xTotal, yTotalErr
		yTotal[Ntotal,Ntotal+Ni-1] = ywi[p-Ntotal]
		if (WaveExists(xwi))
			xTotal[Ntotal,Ntotal+Ni-1] = xwi[p-Ntotal]
		else
			xTotal[Ntotal,Ntotal+Ni-1] = (p-Ntotal)*DimDelta(ywi,0) + DimOffset(ywi,0)
		endif
		Wave ywiErr = GetErrorWave(ywi)					// get error wave if it exists
		if (WaveExists(ywiErr))
			yTotalErr[Ntotal,Ntotal+Ni-1] = ywiErr[p-Ntotal]
		else
			yTotalErr[Ntotal,Ntotal+Ni-1] = NaN		// no errors, set to bad value
		endif
		Ntotal += Ni
	endfor
	if (Ntotal<1)
		return $""												// nothing
	endif
	wnote = ReplaceStringByKey("sourceY",wnote,yList,"=")
	if (strlen(xList))
		wnote = ReplaceStringByKey("sourceX",wnote,xList,"=")
	endif
	wnote = AddClassToWaveNote(wnote,"CombinedData")

	Duplicate/FREE xTotal, xAbs
	xAbs = abs(xTotal)
	WaveStats/M=1/Q xAbs
	WaveClear xAbs
	Redimension/N=(Ntotal+1) yTotal, xTotal, yTotalErr
	yTotal[Ntotal] = yTotal[V_minloc]
	xTotal[Ntotal] = 0
	if (WaveExists(ywiErr))
		yTotalErr[Ntotal] = yTotalErr[V_minloc]
	else
		yTotalErr[Ntotal] = NaN							// no errors, set to bad value
	endif
	Ntotal += 1
	Sort xTotal, xTotal,yTotal, yTotalErr

	WaveStats/M=1/Q yTotalErr
	if (V_npnts)												// there are some valid points
		yTotalErr = numtype(yTotalErr) ? 1 : yTotalErr	// set NaN's to 1
		Wave ywErr = yTotalErr
	else
		Wave ywErr = $""										// no error wave here
	endif

	T_K = (T_K > 0) ? T_K : T_K_default
	wnote = ReplaceNumberByKey("T_Sample",wnote,T_K,"=")
	Note/K yTotal, wnote

	Make/N=3/WAVE/FREE yxWaves
	yxWaves[0] = yTotal
	yxWaves[1] = xTotal
	yxWaves[2] = yTotalErr
	return yxWaves
End
//
Static Function/T MakePhononPeaksUserData(wy,wx,wyErr)
	Wave wy
	Wave wx
	Wave wyErr

	Variable Npeaks=0
	String str, pkName, PhononPeaks="", wName=NameOfWave(wy), loTag,hiTag,loStr="",hiStr=""

	Variable N=DimSize(wy,0)
	Duplicate/FREE wy, wySort
	if (WaveExists(wyErr))
		Duplicate/FREE wyErr, wyErrSort
	else
		Duplicate/FREE wy, wyErrSort
		wyErrSort = sqrt(wyErrSort)
	endif
	Sort wySort, wySort, wyErrSort
	Variable yMid=wySort[floor(N/2)], yErr=wyErrSort[floor(N/2)]
	Variable yThresh = 2*yMid + 3*yErr
	WaveClear wySort, wyErrSort
	// print "yMid =",yMid,"   yThresh =",yThresh

	Duplicate/FREE wy, wyFind
	SetScale/P x 0,1,"", wyFind
	Variable iloMin=Inf, ihiMax=-Inf, aCenter
	Variable pCen, ilo,ihi, xlo,xhi, i
	for (i=0;i<7;i+=1)
		WaveStats/M=1/Q wyFind
		if (V_max<yThresh)
			break									// done, no peaks left
		endif
		pCen = V_maxloc						// add a peak centered at pCen
		ilo = WalkDown2Threshold(wyFind,pCen,yThresh,-1)
		ihi = WalkDown2Threshold(wyFind,pCen,yThresh,1)
		if (!(ilo<pCen && pCen<ihi))		// not a real peak
			wyFind[ilo,ihi] = NaN			// blank this region and search again
			continue
		endif

		ilo = limit(round(pCen - 2*(pCen-ilo)), 0, pCen)
		ihi = limit(round(pCen + 2*(ihi-pCen)), pCen, N-1)
		xlo=wx[ilo]
		xhi=wx[ihi]
		if (numtype(ilo+ihi+xlo+xhi))
			break									// also done
		endif
		wyFind[ilo,ihi] = NaN				// blank this region for next search

		aCenter = (xlo*xhi < 0)
		pkName = SelectString(aCenter, "pk"+num2istr(Npeaks), "pkCen")
		Npeaks += aCenter ? 0 : 1
		sprintf str, "%s:%s,%d,%d,%g,%g;",pkName,wName,ilo,ihi,xlo,xhi
		if (!aCenter && (ilo<iloMin) && xlo<0)	// a new left-most region, and skip center peaks
			sprintf loStr, "%s,%d,%d,%g,%g",wName,0,ihi,wx[0],xhi		// extend to left end
			iloMin = ilo
			loTag = pkName
		endif
		if (!aCenter && (ihi>ihiMax) && xhi>0)	// a new right-most region, and skip center peaks
			sprintf hiStr, "%s,%d,%d,%g,%g",wName,ilo,N-1,xlo,wx[N-1]	// extend to right end
			ihiMax = ihi
			hiTag = pkName
		endif
		PhononPeaks += str
	endfor

	if (Npeaks>1)								// if at least 2 peaks, extend outer ranges to ends
		PhononPeaks = ReplaceStringByKey(loTag,PhononPeaks,loStr)
		PhononPeaks = ReplaceStringByKey(hiTag,PhononPeaks,hiStr)
	endif
	return PhononPeaks
End
//
Static Function WalkDown2Threshold(wy,p0,thresh,direction)
	Wave wy
	Variable p0
	Variable thresh
	Variable direction		// -1=left or +1=right
	direction = direction < 0 ? -1 : 1
	Variable pEnd = direction<0 ? 0 : DimSize(wy,0)-1
	Variable i
	if (direction < 0)
		for (i=p0;i>=0;i+=direction)
			if (wy[i] <= thresh || numtype(wy[i]))
				return i
			endif
		endfor
	else
		for (i=p0;i<=pEnd;i+=direction)
			if (wy[i] <= thresh || numtype(wy[i]))
				return i
			endif
		endfor
	endif
	return pEnd
End



Function ClearAllPhononPeakMarkers(gName)
	String gName					// use "" for top graph
	if (strlen(gName))
		SetWindow $gName userdata(PhononPeaks)=""
	else
		SetWindow kwTopWin userdata(PhononPeaks)=""
	endif
	UpdatePhononMarkersFromUserData(gName)
	return 0
End



// Usually called from the Marquee Menu
Function AddPhononPeakMarker(gName,[xLo,xHi,pLo,pHi,yDataLo,yDataHi])
	String gName						// graph name, use "" for top graph
	Variable xLo,xHi					// x range to use for peak (CANNOT use both points & x)
	Variable pLo,pHi					// point range to use for peak
	Wave yDataLo,yDataHi			// when giving xLo & xHi, need to specify yData too
	xLo = ParamIsDefault(xLo) ? NaN : xLo
	xHi = ParamIsDefault(xHi) ? NaN : xHi
	pLo = ParamIsDefault(pLo) ? NaN : pLo
	pHi = ParamIsDefault(pHi) ? NaN : pHi
	if (WaveExists(yDataLo) && !WaveExists(yDataHi))
		Wave yDataHi = yDataLo
	endif

	if (numtype(pLo+pHi+xLo+xHi)==0)
		print "ERROR -- AddPhononPeakMarker(), cannot used BOTH points and x to set phonon range"
		return 1
	elseif ((numtype(xLo)==0 || numtype(pLo)==0) && !WaveExists(yDataLo))
		print "ERROR -- AddPhononPeakMarker(), cannot specify range without also specifying yData"
		return 1
	elseif ((numtype(xHi)==0 || numtype(pHi)==0) && !WaveExists(yDataHi))
		print "ERROR -- AddPhononPeakMarker(), cannot specify range without also specifying yData"
		return 1
	endif

	if (numtype(pLo+pHi) && numtype(xLo+xHi))	// no lo or no hi, try to use Marquee
		String list=FindPhononPeakMarkerInfo(gName,xLo,xHi,pLo,pHi,yDataLo,yDataHi)
		// example, list = sig,73,31.5662,sig#2,10,49.0128
		pLo = str2num(StringFromList(1,list,","))
		xLo = str2num(StringFromList(2,list,","))
		pHi = str2num(StringFromList(4,list,","))
		xHi = str2num(StringFromList(5,list,","))
	else														// values passed, use them
		DoAlert 0,"in AddPhononPeakMarker(), not yet implemented passing peak positions, only using Marquee"
		return 1
	endif

	if (numtype(pLo+pHi+xLo+xHi))
		return 1
	endif

// uList = 	pkCen:sig,3,-3.42327,sig,17,3.75777;pk0:sig#1,19,34.9399,sig#2,38,43.9991;
	String uList=GetUserData(gName,"","PhononPeaks")
	Variable Npk
	for (Npk=0;Npk<NphononPeaks_MAX;Npk+=1)			// find lowest unused phonon peak marker
		if (strlen(StringByKey("pk"+num2istr(Npk),uList))<1)
			break
		endif
	endfor
	if (Npk>=NphononPeaks_MAX)
		print "Too many phonon peaks identified, cannot add any more"
		return 1
	endif

	String key = SelectString(xLo*xHi>=0, "pkCen", "pk"+num2istr(Npk))
	ulist = ReplaceStringByKey(key,uList,list)

	if (strlen(gName))
		SetWindow $gName userdata(PhononPeaks)=ulist
	else
		SetWindow kwTopWin userdata(PhononPeaks)=ulist
	endif
	UpdatePhononMarkersFromUserData(gName)		// update visible markers on graph
	return 0
End



Function RemovePhononPeakMarker(gName,[xLo,xHi,pLo,pHi,yDataLo,yDataHi])
	String gName						// graph name, use "" for top graph
	Variable xLo,xHi					// x range to use for peak (CANNOT use both points & x)
	Variable pLo,pHi					// point range to use for peak
	Wave yDataLo, yDataHi			// when giving xLo & xHi, need to specify yData too
	xLo = ParamIsDefault(xLo) ? NaN : xLo
	xHi = ParamIsDefault(xHi) ? NaN : xHi
	pLo = ParamIsDefault(pLo) ? NaN : pLo
	pHi = ParamIsDefault(pHi) ? NaN : pHi

	if (numtype(pLo+pHi+xLo+xHi)==0)
		print "ERROR -- RemovePhononPeakMarker(), cannot used BOTH points and x to set phonon range"
		return 1
	elseif ((numtype(xLo)==0 || numtype(pLo)==0) && !WaveExists(yDataLo))
		print "ERROR -- RemovePhononPeakMarker(), cannot specify range without also specifying yData"
		return 1
	elseif ((numtype(xHi)==0 || numtype(pHi)==0) && !WaveExists(yDataHi))
		print "ERROR -- RemovePhononPeakMarker(), cannot specify range without also specifying yData"
		return 1
	endif

	String list=FindPhononPeakMarkerInfo(gName,xLo,xHi,pLo,pHi,yDataLo,yDataHi)
	xLo = str2num(StringFromList(2,list,","))			// x range inside of peak to remove
	xHi = str2num(StringFromList(5,list,","))

	String uList=GetUserData(gName,"","PhononPeaks")
	String pkRemove="", pkName, item
	Variable i,N=ItemsInList(ulist), x0,x1, fit,fitBest=Inf
	for (i=0;i<N;i+=1)
		item = StringFromList(i,ulist,";")				// each peak in uList
		pkName = StringFromList(0,item,":")
		item = StringFromList(1,item,":")
		x0 = str2num(StringFromList(2,item,","))
		x1 = str2num(StringFromList(5,item,","))
		fit = xLo>=x0 && xHi<=x1 ? (xLo-x0) + (x1-xHi) : Inf
		if (fit<fitBest)
			fitBest = fit
			pkRemove = pkName
		endif
	endfor


	if (strlen(pkRemove))
		ulist = RemoveByKey(pkRemove,ulist)
		if (strlen(gName))
			SetWindow $gName userdata(PhononPeaks)=ulist
		else
			SetWindow kwTopWin userdata(PhononPeaks)=ulist
		endif
		UpdatePhononMarkersFromUserData(gName)
		return 0
	else
		return 1
	endif
End


// Usually called from the Marquee Menu
Static Function/S FindPhononPeakMarkerInfo(gName,xLo,xHi,pLo,pHi,yDataLo,yDataHi)
	String gName						// graph name, use "" for top graph
	Variable xLo,xHi					// x range to use for peak (CANNOT use both points & x)
	Variable pLo,pHi					// point range to use for peak
	Wave yDataLo,yDataHi			// when giving xLo & xHi, need to specify yData too (both can be the same)
	if (WaveExists(yDataLo) && !WaveExists(yDataHi))
		Wave yDataHi = yDataLo
	endif

	if (numtype(pLo+pHi+xLo+xHi)==0)
		print "ERROR -- FindPhononPeakMarkerInfo(), cannot used BOTH points and x to set phonon range"
		return ""
	elseif ((numtype(xLo)==0 || numtype(pLo)==0) && !WaveExists(yDataLo))
		print "ERROR -- FindPhononPeakMarkerInfo(), cannot specify range without also specifying yData"
		return ""
	elseif ((numtype(xHi)==0 || numtype(pHi)==0) && !WaveExists(yDataHi))
		print "ERROR -- FindPhononPeakMarkerInfo(), cannot specify range without also specifying yData"
		return ""
	endif

	String trName="", trNameLo="",trNameHi=""
	if (numtype(xLo+xHi) && numtype(pLo+pHi))
		GetMarquee/K/W=$gName left, bottom				// try to get range from Marquee
		if (V_flag)
			gName = S_marqueeWin
			xLo = V_left
			xHi = V_right
		endif
		if (numtype(xLo+xHi))								// Marquee failed, try cursors
			String cinfo=GetCursorRangeFromGraph(gName)
			trName = StringByKey("yData",cinfo,"=")
			Wave yDataLo = TraceNameToWaveRef(gName,trName)
			Wave yDataHi = TraceNameToWaveRef(gName,trName)
			trNameLo = trName
			trNameHi = trName
			pLo = NumberByKey("pLo",cinfo,"=")
			pHi = NumberByKey("pHi",cinfo,"=")
		endif
	endif
	// now have xlo & xhi or plo & phi

	Variable i, widthX=0, Lo,Hi, LoBest=NaN,HiBest=NaN
	if (!WaveExists(yDataLo) || !WaveExists(yDataHi))		// must find the yData wave that covers [xLo,xHi]
		String trNameLowest="",trNameHighest=""
		Variable lowestX=Inf, hightestX=-Inf
		Variable xLoBest=Inf, xHiBest=-Inf
		String list=SpecDataAndXrangeOnGraph(gName), item	// e.g. trace0,Xlo0,Xhi0;trace1,Xlo1,Xhi1;...
		// prefer traces that wholly contains [xLo,xHi]
		for (i=0;i<ItemsInList(list);i+=1)
			item = StringFromList(i,list)				// something like:  "sig#1,25.0,44"
			Lo = str2num(StringFromList(1,item,","))	// low x-value
			Hi = str2num(StringFromList(2,item,","))	// high x-value
			trName = StringFromList(0,item,",")
			if (xLo>=Lo && xHi<=Hi)						// [xLo,xHi] fits all on item, we are done
				trNameLo = trName
				trNameHi = trName
				Wave yDataLo = TraceNameToWaveRef(gName,trName)
				Wave yDataHi = TraceNameToWaveRef(gName,trName)
				break												// found trace that wholly contains [xLo,xHi], done
			endif

			if (strlen(trNameLo)==0 && Lo<xLo && xLo<Hi)	// this trace contains xLo
				trNameLo = trName
			endif
			if (strlen(trNameHi)==0 && Lo<xHi && xHi<Hi)	// this trace contains xHi
				trNameHi = trName
			endif
			if (Lo<lowestX)									// find trace with lowest x value
				trNameLowest = trName
				lowestX = Lo
			endif
			if (HI>hightestX)									// find trace with highest x value
				trNameHighest = trName
				hightestX = Hi
			endif
		endfor
		if (strlen(trNameLo)<1)							// no traces that contain xLo, use the lowest one
			trNameLo = trNameLowest
			xLo = min(xhi,lowestX)
		endif
		if (strlen(trNameHi)<1)							// no traces that contain xHi, use the highest one
			trNameHi = trNameHighest
			xHi = min(xhi,hightestX)
		endif
		Wave yDataLo = TraceNameToWaveRef(gName,trNameLo)
		Wave yDataHi = TraceNameToWaveRef(gName,trNameHi)
	endif
	if (!WaveExists(yDataLo) || !WaveExists(yDataHi))
		return ""
	endif
	Wave xDataLo = XWaveRefFromTrace(gName,trNameLo)
	Wave xDataHi = XWaveRefFromTrace(gName,trNameHi)

	if (numtype(pLo))											// find the low point
		if (WaveExists(xDataLo))							// convert from x to point using xwave
			plo = BinarySearch(xDataLo,xLo)
			plo = plo==-1 ? 0 : plo
		else														// convert from x to point using ywave scaling
			plo = x2pnt(yDataLo,xLo)
			plo = limit(plo,0,numpnts(yDataLo)-1)
		endif
	endif

	if (numtype(pHi))											// find the high point
		if (WaveExists(xDataHi))							// convert from x to point using xwave
			phi = BinarySearch(xDataHi,xHi)
			phi = phi==-2 ? numpnts(yDataHi)-1: phi
			Variable N=DimSize(xDataHi,0)
			if (phi == (N-2))									// special situation for tolerance at end point
				if (abs(xHi-xDataHi[N-1]) < abs(xDataHi[N-1]-xDataHi[N-2])/1000)
					pHi = N-1									// in case xHi is slightly lower than xDataHi[Inf]
				endif
			endif
		else														// convert from x to point using ywave scaling
			phi = x2pnt(yDataHi,xHi)
			phi = limit(phi,0,numpnts(yDataHi)-1)
		endif
	endif
	if (numtype(pLo+pHi))
		return ""
	endif
	sprintf list, "%s,%d,%g,%s,%d,%g",trNameLo,pLo,xLo, trNameHi,pHi,xHi
	return list
End



// from the data in GetUserData(gName,"","PhononPeaks"), update the markers on the graph
//
//				NEW
//	userData:	pkCen:traceCen,pLo,xLo,traceCen,pHi,xHi;pk0:trace0,pLo0,xLo0,trace0,pHi0,xHi0;...
// example:		pk0:sig,76,25.2,sig#1,33,35.6;pkCen:sig,22,-5.05,sig,39,5.1;
//
//				OLD
//	userData:	pkCen:traceCen,pLo,pHi;pk0:trace0,pLo0,pHi0;pk1:trace1,pLo1,pHi1;pk2:trace2,pLo2,pHi2;...
// example:		pk0:yData,76,131;pkCen:yData,206,312;
//
//Static Function UpdatePhononMarkersFromUserData(gName)
Static Function UpdatePhononMarkersFromUserData(gName)
	String gName			// use "" for top graph

	String uList=GetUserData(gName,"","PhononPeaks")

	Make/N=(6,3)/FREE/W/U rgb
	rgb[0][0]= {55000,0,0,0,55000,55000}
	rgb[0][1]= {0,55000,0,55000,0,55000}
	rgb[0][2]= {0,0,55000,55000,55000,0}
	Variable Nc = DimSize(rgb,0)

	String tagName, trNameLo,trNameHi, list, ni, str
	Variable Npk, pLo, pHi, ic, r,g,b
	for (Npk=0;Npk<NphononPeaks_MAX;Npk+=1)
		ni = num2istr(Npk)
		list = StringByKey("pk"+ni,uList)
		trNameLo=StringFromList(0,list,",")
		trNameHi=StringFromList(3,list,",")
		if (strlen(trNameLo))
			Wave ywLo = TraceNameToWaveRef(gName,trNameLo)
		else
			Wave ywLo = $""
		endif
		if (strlen(trNameHi))
			Wave ywHi = TraceNameToWaveRef(gName,trNameHi)
		else
			Wave ywHi = $""
		endif
		pLo = str2num(StringFromList(1,list,","))
		pHi = str2num(StringFromList(4,list,","))
		if (WaveExists(ywLo) && WaveExists(ywHi) && numtype(pLo+pHi)==0)
			pLo = pnt2x(ywLo,pLo)
			pHi = pnt2x(ywHi,pHi)
			ic = mod(Npk,Nc)
			r=rgb[ic ][0];  g=rgb[ic ][1];  b=rgb[ic ][2]
			Tag/W=$gName/C/N=$("PhPeak"+ni+"_Lo")/G=(r,g,b)/X=0/Y=20 $trNameLo, pLo, "-"+ni
			Tag/W=$gName/C/N=$("PhPeak"+ni+"_Hi")/G=(r,g,b)/X=0/Y=20 $trNameHi, pHi, "+"+ni
		else
			Tag/N=$("PhPeak"+ni+"_Lo")/K			// delete the tag for peak i
			Tag/N=$("PhPeak"+ni+"_Hi")/K
		endif
	endfor

	list = StringByKey("pkCen",uList)			// and check for a center peak
	trNameLo=StringFromList(0,list,",")
	trNameHi=StringFromList(3,list,",")
	if (strlen(trNameLo))
		Wave ywLo = TraceNameToWaveRef(gName,trNameLo)
	else
		Wave ywLo = $""
	endif
	if (strlen(trNameHi))
		Wave ywHi = TraceNameToWaveRef(gName,trNameHi)
	else
		Wave ywHi = $""
	endif
	pLo = str2num(StringFromList(1,list,","))
	pHi = str2num(StringFromList(4,list,","))
	if (WaveExists(ywLo) && WaveExists(ywHi) && numtype(pLo+pHi)==0)
		pLo = pnt2x(ywLo,pLo)
		pHi = pnt2x(ywHi,pHi)
		Tag/W=$gName/C/N=$("PhPeakCen_Lo")/G=(0,0,0)/X=0/Y=20 $trNameLo, pLo, "-C"
		Tag/W=$gName/C/N=$("PhPeakCen_Hi")/G=(0,0,0)/X=0/Y=20 $trNameHi, pHi, "+C"
	else
		Tag/N=PhPeakCen_Lo/K						// delete the tag for center peak
		Tag/N=PhPeakCen_Hi/K
	endif

	if (strlen(GetRTStackInfo(2))==0)			// If called from command line, print info
		print "uList = ",uList						// This is mostly for debugging
	endif
	return 0
End


// Get a list of data waves on graph and each ones X range
// returns:   trace0,Xlo0,Xhi0;trace1,Xlo1,Xhi1;trace2,Xlo2,Xhi2;...
//	e.g.:			  sig,-5.01371,40;sig#1,25.0,44;sig#2,44,49.0;
//
// use:	TraceNameToWaveRef(gName,trace0) and XWaveRefFromTrace(gName,trace0)
//				later get the actual waves
Static Function/S SpecDataAndXrangeOnGraph(gName)
	String gName						// graph name, use "" for top graph

	String trListAll=TraceNameList(gName,";",1), trList="", trName
	Variable Nall=ItemsInList(trListAll), N,i
	for (i=0,N=0;i<Nall;i+=1)
		trName = StringFromList(i,trListAll)
		Wave ww = TraceNameToWaveRef(gName,trName)
		if (StringMatch(trName,"fit_*") || strsearch(note(ww),trName,0)==0)	// reject fit results
			continue
		endif
		trList += trName+";"
	endfor

	Variable xLo,xHi
	String list="", str
	for (i=0;i<ItemsInList(trList);i+=1)
		trName = StringFromList(i,trList)
		Wave yw = TraceNameToWaveRef(gName,trName)
		Wave xw = XWaveRefFromTrace(gName,trName)
		if (WaveExists(xw))							// convert from x to point using xwave
			WaveStats/M=1/Q xw
			xLo = V_min
			xHi = V_max
		else												// convert from x to point using ywave scaling
			xLo = pnt2x(yw,0)
			xHi = pnt2x(yw,DimSize(yw,0)-1)
			OrderValues(xLo,xHi)					// ensure xLo < xHi
		endif

		if (numtype(xLo+xHi)==0 && WaveExists(yw))
			sprintf str, "%s,%g,%g;",trName,xLo,xHi
			list += str
		endif
	endfor
	return list
End



// This only works when fitting to waves on a graph that are identified in the "PhononPeaks" userdata
Function/WAVE FitDHarmonicMultipleFromMarkers([showStart,fitBkg,printIt])
	Variable showStart	// only show starting guess, do not fit
	Variable fitBkg		// 1=fit a background,  0=do NOT include a background (default=0)
	Variable printIt		// forces printing
	showStart = ParamIsDefault(showStart) || numtype(showStart) ? 0 : !(!showStart)
	fitBkg = ParamIsDefault(fitBkg) || numtype(fitBkg) ? 0 : !(!fitBkg)
	printIt = ParamIsDefault(printIt) || numtype(printIt) ? strlen(GetRTStackInfo(2))==0 : printit

	if (ParamIsDefault(showStart) && ParamIsDefault(fitBkg) && ParamIsDefault(printIt))
		Prompt showStart, "Fit or just show starting guess", popup, "Fit;Starting Guess Only"
		Prompt fitBkg, "Include a background fit", popup, "NO background;Fit a background"
		showStart = showStart ? 2 : 1
		fitBkg = fitBkg ? 2 : 1
		DoPrompt "What to do", showStart,fitBkg
		if (V_flag)
			return $""
		endif
		showStart = showStart==2
		fitBkg = fitBkg==2
		printIt = 1
	endif
	fitBkg = fitBkg ? 1 : 0									// fitBkg MUST be 0 or 1 ONLY

	String gName=""
	String uList=GetUserData(gName,"","PhononPeaks"), list
	// example:		pkCen:sig,3,-3.42327,sig,19,4.91444;pk0:sig,76,33.0121,sig#2,10,49.0128;pk1:sig,61,25.7346,sig,72,31.3252;
	Variable m, centralPeak=0, Npks		// Npks is number of peaks to fit (not number of phonons)
	String pkNames=""							// list of all peak labels present in uList, pkCen,pk0,pk1,...
	for (m=0,Npks=0; m<NphononPeaks_MAX; m+=1)
		if (strlen(StringByKey("pk"+num2istr(m),uList)))
			pkNames += "pk"+num2istr(m)+";"
			Npks += 1
		endif
	endfor
	if (Npks<1)
		if (printIt)
			print "ERROR -- FitDHarmonicMultipleFromMarkers(), cannot find any phonon peaks"
		endif
		return $""								// no phonon peaks to fit
	endif
	if (strlen(StringByKey("pkCen",uList)))
		pkNames += "pkCen"					// NOTE, pkCen will ALWAYS be the last one in the list
		centralPeak = 1
		Npks += 1
	endif


	Make/N=(Npks,2)/D/FREE xLoHi=NaN						// range for each phonon peak
	Variable i, xLoEnd=Inf, xHiEnd=-Inf, xvalue
	for (i=0;i<ItemsInList(pkNames);i+=1)		// find the maximal x-range of peaks in uList [xLoEnd,xHiEnd]
		list = StringByKey(StringFromList(i,pkNames),uList)
		xvalue = str2num(StringFromList(2,list,","))
		xLoHi[i][0] = xvalue
		xLoEnd = min(xLoEnd,xvalue)
		xHiEnd = max(xHiEnd,xvalue)
		xvalue = str2num(StringFromList(5,list,","))
		xLoHi[i][1] = xvalue
		xLoEnd = min(xLoEnd,xvalue)
		xHiEnd = max(xHiEnd,xvalue)
	endfor
	if (printIt)
		printf "x range = [%g, %g]\r",xLoEnd,xHiEnd
	endif
	if (numtype(xLoEnd+xHiEnd))
		if (printIt)
			print "ERROR -- FitDHarmonicMultipleFromMarkers(), cannot find the total x-range"
		endif
		return $""
	endif
	// will be fitting all measured values in range [xLoEnd,xHiEnd], also now have array xLoHi[][2]


	// find all traces to use, i.e. all traces that have data in the range [xLoEnd,xHiEnd]
	String allTraces=TraceNameList(gName,";",1), trName,trList=""		// all traces with data in [xLoEnd,xHiEnd]
	Variable T_K=NaN, xLo,xHi, Ntraces
	for (i=0,Ntraces=0; i<ItemsInList(allTraces); i+=1)
		trName = StringFromList(i,allTraces)
		if (WhichListItem(trName,trList)>=0)			// trName already in trList, skip it
			continue
		endif
		// check if trName covers [xLoEnd,xHiEnd]
		Wave ywi = TraceNameToWaveRef(gName,trName)
		Wave xwi = XWaveRefFromTrace(gName,trName)
		if (!WaveInClass(ywi,"specWave*"))				// only try to fit spec waves
			continue
		endif
		if (WaveExists(xwi))
			WaveStats/Q/M=1 xwi
			xLo = V_min
			xHi = V_max
		else
			xLo = DimOffset(ywi,0)
			xHi = DimOffset(ywi,0) + DimDelta(ywi,0)*(DimSize(ywi,0)-1)
			if (xHi<xLo)
				Variable swap
				swap = xLo
				xLo = xHi
				xHi = swap
			endif
		endif
		if (xHi>=xLoEnd || xLo<=xHiEnd)					// this wave spans part of [xLoEnd,xHiEnd]
			trList += trName + ";"							// add it to list
			if (Ntraces==0)
				String fldr=GetWavesDataFolder(ywi,1)
			endif
			Ntraces += 1
		endif
	endfor
	// trList, now contains all of the traces that span my range of interest [xLoEnd,xHiEnd]


	// create the waves yw & xw, the data to fit, and set T_K
	if (Ntraces<1)
		if (printIt)
			printf "ERROR --FitDHarmonicMultipleFromMarkers(), cannot determine number of waves"
		endif
		return $""
	elseif (Ntraces==1)										// only one wave
		trName = StringFromList(0,trList)
		Wave yw = TraceNameToWaveRef(gName,trName)
		Wave xw = XWaveRefFromTrace(gName,trName)
		Wave ywErr = GetErrorWave(yw)		// get error wave (if it exists)
		T_K = T_SampleFromYwave(yw,NaN)
	else
		Make/N=0/D/FREE yTotal, xTotal, yTotalErr=NaN	// always a point at 0
		String yList="", xList=""
		Variable Ni,Ntotal
		for (i=0,Ntotal=0; i<Ntraces; i+=1)
			trName = StringFromList(i,trList)
			Wave ywi = TraceNameToWaveRef(gName,trName)
			Wave xwi = XWaveRefFromTrace(gName,trName)
			Ni = numpnts(ywi)
			if (Ni<1)
				continue												// empty
			endif
			yList += fwhm#prettyWaveName(ywi)+";"
			xList += fwhm#prettyWaveName(xwi)+";"
			T_K = T_K>0 ? T_K : T_SampleFromYwave(ywi,NaN)

			Redimension/N=(Ntotal+Ni) yTotal, xTotal, yTotalErr
			yTotal[Ntotal,Ntotal+Ni-1] = ywi[p-Ntotal]
			if (WaveExists(xwi))
				xTotal[Ntotal,Ntotal+Ni-1] = xwi[p-Ntotal]
			else
				xTotal[Ntotal,Ntotal+Ni-1] = (p-Ntotal)*DimDelta(ywi,0) + DimOffset(ywi,0)
			endif
			Wave ywiErr = GetErrorWave(ywi)	// get error wave if it exists
			if (WaveExists(ywiErr))
				yTotalErr[Ntotal,Ntotal+Ni-1] = ywiErr[p-Ntotal]
			else
				yTotalErr[Ntotal,Ntotal+Ni-1] = NaN		// no errors, set to bad value
			endif
			Ntotal += Ni
		endfor
		T_K = (T_K > 0) ? T_K : T_K_default

		Duplicate/FREE xTotal, xAbs
		xAbs = abs(xTotal)
		WaveStats/M=1/Q xAbs
		WaveClear xAbs
		Redimension/N=(Ntotal+1) yTotal, xTotal, yTotalErr
		yTotal[Ntotal] = yTotal[V_minloc]					// make an extra point at the minimum location and set to 0???
		xTotal[Ntotal] = 0
		if (WaveExists(ywiErr))
			yTotalErr[Ntotal] = yTotalErr[V_minloc]		// set the error for this extra point
		else
			yTotalErr[Ntotal] = NaN							// no errors, set to bad value
		endif
		Ntotal += 1

		Sort xTotal, xTotal,yTotal, yTotalErr
		Make/N=(Ntotal)/O $(fldr+"fit_multi")/Wave=fit_W		// used to view the fit
		Make/N=(Ntotal)/O $(fldr+"fitX_multi")/Wave=fitX_W
		fit_W = NaN
		fitX_W = xTotal
		uList = ReplaceNumberByKey("T_Sample",uList,T_K,"=")
		Note/K yTotal, uList
		Note/K fit_W, uList

		Wave yw=yTotal, xw=xTotal

		WaveStats/M=1/Q yTotalErr
		if (V_npnts)												// there are some valid points
			yTotalErr = numtype(yTotalErr) ? 1 : yTotalErr	// set NaN's to 1
			Wave ywErr = yTotalErr
		else
			Wave ywErr = $""										// no error wave here
		endif
	endif
	Variable pLo = x2pntWyWx(yw,xw,xLoEnd)				// convert from x to point
	Variable pHi = x2pntWyWx(yw,xw,xHiEnd)

	// find number of phonons (not peaks), look for overlap of the mid points
	Variable Nph=0, j, mid
	Make/N=(Npks,2)/FREE mates=-1
	Make/N=(Npks)/FREE pkUsed=0
	for (i=0;i<Npks-1;i+=1)
		if (pkUsed[i])
			continue
		elseif (xLoHi[i][0]*xLoHi[i][1] < 0)	// a central peak, not a phonon
			pkUsed[i] = 1
			continue
		endif
		Nph += 1											// found a new phonon
		mates[Nph-1][0] = i
		pkUsed[i] = 1
		mid = (xLoHi[i][0] + xLoHi[i][1])/2	// mid point of this phonon
		for (j=i+1;j<Npks;j+=1)					// check all the following peaks
			if (pkUsed[j])
				continue
			elseif (-mid == limit(-mid,xLoHi[j][0],xLoHi[j][1]))	// matches a peak on other side, same phonon
				mates[Nph-1][1] = j												// this peak is a mate to peak at [i]
				pkUsed[j] = 1
			elseif (mid == limit(mid,xLoHi[j][0],xLoHi[j][1]))		// matches a peak on same side, a duplicate
				pkUsed[j] = 1
			endif
		endfor
	endfor
	if (Nph<1)
		return $""
	endif
	Redimension/N=(Nph,2) mates

	Variable useP = numtype(pLo+pHi) ? 0 : 1			// pLo & pHi are valid numbers
	useP = useP && ( pLo>0 || pHi<(DimSize(yw,0)-1))	// pLo & pHi do not extend past range of points

	String units, fldrSav0=GetDataFolder(1)
	if (WaveExists(xw))
		units = WaveUnits(xw,-1)
	else
		units = WaveUnits(yw,0)
	endif
	if (printIt)
		String xname
		if (WaveExists(xw))
			xname = GetWavesDataFolder(xw,2)
		else
			xname = "$\"\""
		endif
		printf "FitDHarmonicMultipleFromMarkers(%s,%s",GetWavesDataFolder(yw,2),xname
		if (!ParamIsDefault(showStart) || showStart)
			printf ", showStart=%g",showStart
		endif
		if (!ParamIsDefault(fitBkg) || fitBkg)
			printf ", fitBkg=%g",fitBkg
		endif
		printf ")"
		if (!useP)
			printf "\t\t// xLo=%g,xHi=%g",xLo,xHi
		else
			printf "\t\t// pLo=%g,pHi=%g",pLo,pHi
		endif
		if (WaveType(yw,2)==2)
			printf "    y-waves = %s\r",yList
			if (strlen(StringByKey("yWaveList",note(yw),"=")))
				printf "    x-waves = %s\r",xList
			endif
		endif
	endif

	pLo = limit(pLo,0,DimSize(yw,0)-1)
	pHi = limit(pHi,0,DimSize(yw,0)-1)
	if (!WaveExists(yw) || numtype(pLo+pHi)==2)
		if (printIt)
			print "ERROR -- FitDHarmonicMultipleFromMarkers(), Cannot find data waves from top graph"
		endif
		return $""
	elseif (xHiEnd*xLoEnd >= 0)
		if (printIt)
			print "ERROR -- FitDHarmonicMultipleFromMarkers(), selected range does not span origin"
		endif
		return $""
	endif


	Make/N=(Npks,2)/D/FREE pLoHi=NaN						// point range for fitting each phonon peak
	for (i=0;i<Npks;i+=1)
		pLoHi[i][0] = x2pntWyWx(yw,xw,xLoHi[i][0])
		pLoHi[i][1] = x2pntWyWx(yw,xw,xLoHi[i][1])
	endfor
	//Duplicate/O yw, ywView
	//Duplicate/O xw, xwView
	//Duplicate/O pLoHi, pLoHiView

	Variable icp = centralPeak ? 1 : 0
	Make/N=(1+icp+3*Nph+fitBkg)/O/D $(fldr+"Coefs")/WAVE=cw = NaN	// Eo and 3 numbers for each phonon, with optional bkg
	Variable ibkg = fitBkg ? icp+3*Nph+1 : -1	// index for bkg in cw
	Variable i0,i1
	for (i=0;i<Nph;i+=1)
		i0 = mates[i][0]
		Wave Wc0 = CalcSimplePeakParameters(yw,xw,1,pLo=pLoHi[i0][0],pHi=pLoHi[i0][1])	// Wc={bkg, amp, center, FWHM, net, COM}
		cw[3*i+1+icp] = peakHeight2phononAmp(Wc0[2],Wc0[3],T_K,Wc0[1])	// amp
		cw[3*i+2+icp] = Wc0[2]							// phonon energy
		cw[3*i+3+icp] = FWHM2gammaPhonon(Wc0[2],Wc0[3])	// gamma, was Wc0[3]=fwhm
		i1 = mates[i][1]
		if (i1>0)
			Wave Wc1 = CalcSimplePeakParameters(yw,xw,1,pLo=pLoHi[i1][0],pHi=pLoHi[i1][1])
			if (numtype(cw[0]))							// Eo is bad, set it
				cw[0] = (Wc0[2]+Wc1[2])/2
			endif
			cw[3*i+2+icp] = abs(Wc0[2]-Wc1[2])/2	// phonon energy
		endif
	endfor
	if (fitBkg)
		cw[ibkg] = smallestNonZeroValue(yw)		// bakcground starting point
	endif

	if (centralPeak)
		i = Npks-1
		Wave Wc0 = CalcSimplePeakParameters(yw,xw,1,pLo=pLoHi[i][0],pHi=pLoHi[i][1])
		cw[1] = Wc0[1]										// amplitude
		cw[0] = Wc0[2]
		cw[1] = cw[1]==0 ? 0.1 : cw[1]
	endif
	cw[0] = numtype(cw[0]) ? 0 : cw[0]				// ensure reasonable Eo
	Note/K cw, "waveClass=DampedHarmonicCoefs;T_Sample="+num2str(T_K)+";"		// pass sample T (K) in wave note

	String str
	SetDimLabel 0, 0, Eo,cw
	if (icp)
		SetDimLabel 0, 1, CenterAmp, cw
	endif
	for (i=0;i<Nph;i+=1)
		str = "amp"+num2istr(i+1)		;	SetDimLabel 0, 1+icp+i*3, $str, cw
		str = "Xpeak"+num2istr(i+1)	;	SetDimLabel 0, 2+icp+i*3, $str, cw
		str = "gamma"+num2istr(i+1)	;	SetDimLabel 0, 3+icp+i*3, $str, cw
	endfor
	if (fitBkg)
		SetDimLabel 0, ibkg, bkg, cw
	endif

	String fitName=SelectString(centralPeak,"DampedHarmonicResWave","DampedHarmonicResWaveCPk")
	if (printIt)
		printf "	Starting Fit with coefficients \"%s\" = %s\r",GetWavesDataFolder(cw,2),vec2str(cw)
		printf "    and using T = %g K,   fitting to:  \"%s\"\r",T_K, fitName
	endif
	String startCW=vec2str(cw,sep=",")

	Wave resolutionWave=root:resolutionWave		// ensure that the resolution wave exists
	if (!WaveExists(resolutionWave))
		Wave resolutionWave=herix#SetResolutionWave(yw)
	endif
	if (!WaveExists(resolutionWave))
		DoAlert 0, "Unable to get resolution wave"
		return $""
	endif
	Init_DampedHarmonic()

	Variable imid
	if (WaveExists(xw))									// x-wave given
		imid = BinarySearchInterp(xw,0)
		imid = round(imid)
	else														// y-wave is assumed scaled
		imid = -DimOffset(yw,0)/DimDelta(yw,0)
	endif

	Duplicate/FREE yw, weight
	if (WaveExists(ywErr))
		weight = ywErr[p]>0 ? 1/ywErr[p] : 1		// weight is 1/error
	elseif (isPositiveInts(yw))						// assuming that yw is counts, so root(N)
		weight = yw[p]<1 ? 1 : 1/sqrt(yw[p])		// for counts, sigma=sqrt(yw), a last desperate attempt
	else
		weight = 1											// not counts, just set all weights to 1
	endif

	if (WaveExists(fit_W))
		CheckDisplayed/W=$gName fit_W
		if (!V_flag)
			AppendToGraph/W=$gName fit_W vs fitX_W
		endif
	endif

	FUNCREF DampedHarmonic fitFunc = $fitName
	SetDataFolder fldr
	Variable err=0, V_FitError=0, V_FitQuitReason=0
	if (showStart)
		if (WaveExists(fit_W))
			FuncFit/Q/NTHR=0/O fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D=fit_W/NWOK
		else
			FuncFit/Q/NTHR=0/O fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D/NWOK
		endif
	else
		if (WaveExists(fit_W))
			FuncFit/Q/NTHR=0 fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D=fit_W/NWOK
		else
			FuncFit/Q/NTHR=0 fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D/NWOK
		endif
	endif
	err = V_FitError || V_FitQuitReason
	for (i=0;i<Nph;i+=1)
		cw[2+icp+3*i] = abs(cw[p])					// E(phonon) always positive
		cw[3+icp+3*i] = abs(cw[p])					// gamma always positive
	endfor
	cw[1] = centralPeak ? abs(cw[1]) : cw[1]

	Wave W_sigma=W_sigma
	if (V_FitError || V_FitQuitReason)
		if (printIt)
			print "Fit Failed ************************"
		endif
		if (printIt)
			print FitErrorString(V_FitError,V_FitQuitReason)
		endif
	endif
	SetDataFolder fldrSav0

	if (printIt && !err)
		if (showStart)
			Duplicate/FREE cw, sigma
			sigma = 0										// no error in the guess
		else
			Wave sigma = W_sigma
		endif

		if (centralPeak)
			printf "	Herix Eo	= %s meV,  CentralAmp = %s     chi^2 = %.5g",ValErrStr(cw[0],sigma[0],sp=1),ValErrStr(cw[1],sigma[1],sp=1),V_chisq
		else
			printf "	Herix Eo	= %s meV,     chi^2 = %.5g",ValErrStr(cw[0],sigma[0],sp=1),V_chisq
		endif
		if (fitBkg)
			printf ",  bkg = %s\r",ValErrStr(cw[ibkg],sigma[ibkg],sp=1)
		else
			printf "\r"
		endif

		String tStr = SelectString(showStart,"Fit to peak(s):\r","Starting guess:\r")
		sprintf str, "%s\S2\M\t= %g\r", Letter2SymbolorUnicode("chi"), roundSignificant(V_chisq,3)
		tStr += str
		sprintf str, "E\\Bo\\M\t= %s\r", ValErrStr(cw[0],sigma[0],sp=1)
		tStr += str
		if (icp)
			sprintf str, "amp\t= %s\r", ValErrStr(cw[1],sigma[1],sp=1)
			tStr += str
		endif
		if (fitBkg)
			sprintf str, "bkg\t= %s\r", ValErrStr(cw[ibkg],sigma[ibkg],sp=1)
			tStr += str
		endif
		tStr += "\r"
		for (i=0;i<Nph;i+=1)
			printf " E(phonon) = %s meV,   gamma = %s meV,  amp = %s\r",ValErrStr(cw[2+icp+i*3],sigma[2+icp+i*3],sp=1),ValErrStr(cw[3+icp+i*3],sigma[3+icp+i*3],sp=1),ValErrStr(cw[1+icp+i*3],sigma[1+icp+i*3],sp=1)
			tStr += SelectString(i>=1,"","\r\r")
			sprintf str, "E\\Bph\\M\t= %s\r", ValErrStr(cw[2+icp+i*3],sigma[2+icp+i*3],sp=1)
			tStr += str
			sprintf str, "%s\t= %s\r", Letter2SymbolorUnicode("gamma"), ValErrStr(cw[3+icp+i*3],sigma[3+icp+i*3],sp=1)
			tStr += str
			sprintf str, "amp\t= %s", ValErrStr(cw[1+icp+i*3],sigma[1+icp+i*3],sp=1)
			tStr += str
		endfor
	endif

	String wnote="waveClass=DampedHarmonicCoefs;"
	if (WaveType(yw,2)==2)
		wnote = ReplaceStringByKey("sourceY",wnote,ReplaceString(";",yList,","),"=")
		if (WaveExists(xw))
			wnote = ReplaceStringByKey("sourceX",wnote,ReplaceString(";",xList,","),"=")
		endif
	else
		wnote = ReplaceStringByKey("sourceY",wnote,GetWavesDataFolder(yw,2),"=")
		if (WaveExists(xw))
			wnote = ReplaceStringByKey("sourceX",wnote,GetWavesDataFolder(xw,2),"=")
		endif
	endif
	if (!WaveExists(fit_W) && useP)
		sprintf str,"%g,%g",pLo,pHi
		wnote = ReplaceStringByKey("pRange",wnote,str,"=")
	elseif (WaveExists(fit_W) && numtype(xLo+xHi)==0)
		sprintf str,"%g,%g",xLo,xHi
		wnote = ReplaceStringByKey("xRange",wnote,str,"=")
	endif
	wnote = ReplaceStringByKey("function",wnote,fitName,"=")
	wnote = ReplaceStringByKey("startingCoefs",wnote,startCW,"=")
	if (WaveExists(resolutionWave))
		wnote = ReplaceStringByKey("resolutionWave",wnote,GetWavesDataFolder(resolutionWave,2),"=")
	endif
	if (!err)
		wnote = ReplaceNumberByKey("chisq",wnote,roundSignificant(V_chisq,5),"=")
		wnote = ReplaceStringByKey("W_sigma",wnote,vec2str(sigma,sep=","),"=")
	endif
	Note/K cw, wnote

	CheckDisplayed/W=$gName yw, fit_W
	if (V_flag && strlen(tStr))
#if (IgorVersion()<7)
		TextBox/C/N=textFitDamped/B=1/F=0/Y=3 tStr
#else
		TextBox/C/N=textFitDamped/B=(65535,65535,65535,50000)/F=0/Y=3 tStr
#endif
	endif
	return cw
End



Static Function/WAVE GetErrorWave(yw)
	Wave yw			// find the error wave that goes with yw

	String fldr = GetWavesDataFolder(yw,1)	// only look in same data folder as yw

	// first look in the wave note of yw
	String errName = fldr+StringByKey("ErrorWave",note(yw),"=")
	if (exists(errName)==1)						// wave exists, use it
		return $errName
	endif

	// try looking for same wave with "_err" appended in same data folder as yw
	errName = fldr+NameOfWave(yw)+"_err"
	if (exists(errName)==1)						// wave exists, use it
		return $errName
	endif
	return $""											// give up, no error wave
End




// This is the fitting function when using a resolution wave and a Central peak
Function DampedHarmonicResWaveCPk(coef,meV) : FitFunc
	Wave coef					//	= {Eo, CenterPeakAmp, amp, phonon, gamma}
	Variable meV

	// bkg is zero unless the label of the last element in coef[] is "bkg"
	Variable bkg = StringMatch(GetDimLabel(coef,0,DimSize(coef,0)-1),"bkg") ? coef[DimSize(coef,0)-1] : 0

	Wave resolutionWave = root:resolutionWave // measured resolution
	Duplicate/FREE resolutionWave, Vsum
	Vsum = resolutionWave(x) * DampedHarmonicCPk(coef,x+meV)
	Variable out = sum(Vsum)

	meV = limit(meV,leftx(resolutionWave),rightx(resolutionWave)-deltax(resolutionWave))
	Variable pkValue=WaveMax(resolutionWave)
	Variable xval=limit(meV-coef[0],leftx(resolutionWave),rightx(resolutionWave)-deltax(resolutionWave))
	out += abs(coef[1]) * resolutionWave(xval)/pkValue		// always positive
	return out + bkg
End
//
// This function is ONLY to be used with DampedHarmonicResWaveCPk
//	although it does not use a resolution wave, it makes no sense to use it without one.
Static Function DampedHarmonicCPk(coef,meV)
	Wave coef					// = {Eo, CenterPeakAmp, amp, phonon, gamma}
	Variable meV

	Variable T_K = NumVarOrDefault("root:Packages:DampedHarmonic:T_K",NaN)	// try OVERRIDE value
	T_K = T_K>0 ? T_K : NumberByKey("T_Sample",note(coef),"=")
	T_K = T_K>0 ? T_K : T_K_default
	meV -= coef[0]		// shift energy by Herix offset
	Variable i, Nph=floor((DimSize(coef,0)-1)/3), total=0
	if (abs(meV/(kB*T_K)) < 1e-10)		// for small meV, use a linear expansion about zero.
		for (i=0,total=0; i<Nph; i+=1)
			total += abs(coef[3*i+2]) * (meV + kB*T_K) * abs(coef[3*i+4])/( (coef[3*i+3])^4 )
		endfor
	else
		Variable B_E = abs(1 + 1/(exp(meV/(kB*T_K))-1))	// Bose-Einstein occupation {1 + n(meV)}
		for (i=0,total=0; i<Nph; i+=1)
			total += abs(coef[3*i+2]) * B_E * abs(coef[3*i+4]*meV) / ( (meV*meV-coef[3*i+3]*coef[3*i+3])^2 + (coef[3*i+4]*meV)^2 )
		endfor
	endif
	return total
End

//  ======================== End of Multi-Phonon Interactive Fit =========================  //
//  ======================================================================================  //




//  ======================================================================================  //
//  ============================== Start of Multi-Scan Fit ===============================  //

Function SetCentralExclusionHWE(hw,[printIt])
	Variable hw				// in meV
	Variable printIt
	printIt = ParamIsDefault(printIt) ? NaN : printIt
	printIt = numtype(printIt)  ? strlen(GetRTStackInfo(2))<1 : !(!printIt)

	Variable hwDefault = NumVarOrDefault("root:Packages:DampedHarmonic:excludeHWE",excludeHWdefaultE)
	if (hw<0 || numtype(hw))								// bad value, ask user
		hw = hwDefault
		Prompt hw,"Exclude  meV about Zero"
		DoPrompt "Exclude HW (meV)", hw
		if (V_flag)
			return hwDefault
		endif
		printIt = 1
	endif
	if (printIt)
		printf "SetCentralExclusionHWE(%g)\r",hw
	endif

	if (hw>=0 && numtype(hw)==0)							// valid input
		Init_DampedHarmonic()													// ensure data folders exists
		Variable/G root:Packages:DampedHarmonic:excludeHWE = hw	// good value, set it
	else
		hw = hwDefault
	endif
	return hw
End




Function/WAVE FitDampedHarmonicMultiple([Nph,cw,useResolution,xLo,xHi,pLo,pHi,showStart,printIt])
	Variable Nph			// number of phonons (default is 1, yw has 2 peaks symmetric about 0 eV)
	Wave cw					// optional input wave to receive coeffs
	Variable useResolution
	Variable xLo,xHi		// x range to fit
	Variable pLo,pHi		// point range to fit
	Variable showStart	// only show starting guess, do not fit
	Variable printIt		// forces printing

	Variable NphLast = NumVarOrDefault("root:Packages:DampedHarmonic:NphLast",1)
	Nph = ParamIsDefault(Nph) ? NaN : round(Nph)
	Nph = numtype(Nph) || Nph<1 ? NaN : Nph
	if (ParamIsDefault(cw))
		Wave cw = $""
	endif
	useResolution = ParamIsDefault(useResolution) ? NaN : !(!useResolution)
	useResolution = numtype(useResolution) ? NaN : useResolution
	xLo = ParamIsDefault(xLo) || numtype(xLo) ? -Inf : xLo
	xHi = ParamIsDefault(xHi) || numtype(xHi) ? Inf : xHi
	pLo = ParamIsDefault(pLo) || numtype(pLo) ? -Inf : pLo
	pHi = ParamIsDefault(pHi) || numtype(pHi) ? Inf : pHi
	showStart = ParamIsDefault(showStart) ? 0 : showStart
	showStart = numtype(showStart) ? 0 : !(!showStart)
	printIt = ParamIsDefault(printIt) ? NaN : !(!printIt)
	printIt = numtype(printIt) ? strlen(GetRTStackInfo(2))<1 : printIt
	if (numtype(pLo+xLo)==0 || numtype(pHi+xHi)==0)
		print "ERROR -- FitDampedHarmonicMultiple(), cannot specify both point range and x range on input"
		return $""
	endif

	Nph = (WaveExists(cw) && numtype(Nph)) ? floor((DimSize(cw,0)-1)/3) : Nph
	if (Nph<1 || numtype(Nph) || numtype(useResolution))
		useResolution = useResolution || numtype(useResolution) ? 1 : 2
		Nph = numtype(Nph) || Nph<1 ? NphLast : Nph
		Prompt useResolution,"Use Measured Resolution", popup,"Use Resolution;Do NOT use Resolution"
		Prompt Nph,"Number of phonons to find", popup,"1 phonon pair;2 phonon pairs;3 phonon pairs;4 phonon pairs"
		Prompt showStart, "Fit or just show starting guess", popup, "Fit;Starting Guess Only"
		showStart = showStart ? 2 : 1
		DoPrompt "Number of Phonons",Nph, useResolution, showStart
		if (V_flag)
			return $""
		endif
		useResolution = useResolution==1
		showStart = showStart==2
		printIt = 1
	endif
	if (Nph<1 || numtype(Nph) || numtype(useResolution))
		return $""
	endif

	Variable T_K=NaN
	String fitInfo = WhatToFitInfoMulti("",xLo=xLo,xHi=xHi)
	String fldr=StringByKey("fldr",fitInfo,"=")
	String gName=StringByKey("graphName",fitInfo,"=")
	xLo = numtype(xLo) ? NumberByKey("xLo",fitInfo,"=") : xLo
	xHi = numtype(xHi) ? NumberByKey("xHi",fitInfo,"=") : xHi
	Wave yw = $(fldr+StringByKey("yWave",fitInfo,"="))
	if (WaveExists(yw))											// just a single wave to fit, not multiple waves
		Wave xw = $(fldr+StringByKey("xWave",fitInfo,"="))
		pLo = NumberByKey("pLo",fitInfo,"=")
		pHi = NumberByKey("pHi",fitInfo,"=")
		T_K = T_SampleFromYwave(yw,NaN)
	else 																// multiple waves to fit, make the wave (maybe x-wave too)
		String yList=StringByKey("yWaveList",fitInfo,"="), xList=StringByKey("xWaveList",fitInfo,"=")
		Make/N=0/D/FREE yTotal, xTotal						// always a point at 0
		Variable i,Ni,Ntotal
		for (i=0,Ntotal=0; i<ItemsInList(yList,","); i+=1)
			Wave ywi = $StringFromList(i,yList,",")
			Wave xwi = $StringFromList(i,xList,",")
			if (!WaveExists(ywi))
				continue
			endif
			Ni = numpnts(ywi)
			if (Ni<1)
				continue
			endif
			T_K = (T_K > 0) ? T_K : T_SampleFromYwave(yw,NaN)	// if T_K is NOT valid, try to get it again
			Redimension/N=(Ntotal+Ni) yTotal, xTotal
			yTotal[Ntotal,Ntotal+Ni-1] = ywi[p-Ntotal]
			if (WaveExists(xwi))
				xTotal[Ntotal,Ntotal+Ni-1] = xwi[p-Ntotal]
			else
				xTotal[Ntotal,Ntotal+Ni-1] = (p-Ntotal)*DimDelta(ywi,0) + DimOffset(ywi,0)
			endif
			Ntotal += Ni
		endfor
		Duplicate/FREE xTotal, xAbs
		xAbs = abs(xTotal)
		WaveStats/M=1/Q xAbs
		WaveClear xAbs
		Redimension/N=(Ntotal+1) yTotal, xTotal
		yTotal[Ntotal] = yTotal[V_minloc]
		xTotal[Ntotal] = 0
		Ntotal += 1

		Sort xTotal, xTotal,yTotal
		Make/N=(Ntotal)/O $(fldr+"fit_multi")/Wave=fit_W		// used to view the fit
		Make/N=(Ntotal)/O $(fldr+"fitX_multi")/Wave=fitX_W
		fitX_W = xTotal
		if (T_K>0)
			fitInfo = ReplaceNumberByKey(fitInfo,fitInfo,T_K,"=")
		endif
		Note/K yTotal, fitInfo
		Note/K fit_W, fitInfo

		Wave yw=yTotal, xw=xTotal
		pLo = numtype(xLo) ? -Inf : BinarySearch(xw,xLo)
		pHi = numtype(xHi) ? Inf : BinarySearch(xw,xHi)
	endif
	T_K = (T_K > 0) ? T_K : T_K_default

	Variable useP = numtype(pLo+pHi) ? 0 : 1
	if (useP)
		useP = pLo>0 || pHi<DimSize(yw,0)-1
	endif

	String fldrSav0= GetDataFolder(1)

	String units = SelectString(WaveExists(xw),WaveUnits(yw,0),WaveUnits(xw,-1))
	if (printIt)
		String xname = SelectString(WaveExists(xw),"$\"\"",GetWavesDataFolder(xw,2))
		printf "FitDampedHarmonicMultiple(%s,%s",GetWavesDataFolder(yw,2),xname
		if (!ParamIsDefault(cw))
			printf ", cw=%s",GetWavesDataFolder(cw,2)
		endif
		if (!ParamIsDefault(useResolution) || useResolution)
			printf ", useResolution=%g",useResolution
		endif
		if (numtype(xLo)==0)
			printf ", xLo=%g",xLo
		endif
		if (numtype(xHi)==0)
			printf ", xHi=%g",xHi
		endif
		printf ")"
		if (useP)
			printf "\t\t// pLo=%g,pHi=%g",pLo,pHi
		endif
		printf "\r"
		if (WaveType(yw,2)==2)
			printf "    y-waves = %s\r",StringByKey("yWaveList",note(yw),"=")
			if (strlen(StringByKey("yWaveList",note(yw),"=")))
				printf "    x-waves = %s\r",StringByKey("xWaveList",note(yw),"=")
			endif
		endif
	endif

	pLo = limit(pLo,0,DimSize(yw,0)-1)
	pHi = limit(pHi,0,DimSize(yw,0)-1)
	if (!WaveExists(yw) || numtype(pLo+pHi)==2)
		print "ERROR -- FitDampedHarmonicMultiple(), Cannot find data waves from top graph"
		return $""
	elseif (xHi*xLo >= 0)
		print "ERROR -- FitDampedHarmonicMultiple(), selected range does not span origin"
		return $""
	endif

	if (!WaveExists(cw))
		SetDataFolder fldr
		Wave cw = SetDampedHarmonicCoefsE(yw,xw,Nph=Nph)
		SetDataFolder fldrSav0
	endif
	if (!WaveExists(cw))
		print "ERROR -- FitDampedHarmonicMultiple(), Could not get the Coefficients wave"
		return $""
	endif

	Variable excludeHWE = NumVarOrDefault("root:Packages:DampedHarmonic:excludeHWE",excludeHWdefaultE)
	if (printIt)
		printf "	Starting Fit with coefficients \"%s\" = %s\r",GetWavesDataFolder(cw,2),vec2str(cw)
		printf "    and using T = %g K,   Excluded HW = %g meV\r",T_K, excludeHWE
	endif
	String startCW=vec2str(cw,sep=",")

	Wave resolutionWave=root:resolutionWave				// ensure that the resolution wave exists
	if (!WaveExists(resolutionWave))
		Wave resolutionWave=herix#SetResolutionWave(yw)
	endif
	if (!WaveExists(resolutionWave))
		DoAlert 0, "Unable to get resolution wave"
		return $""
	endif
	Init_DampedHarmonic()
	Variable/G root:Packages:DampedHarmonic:NphLast = Nph

	Variable imid
	if (WaveExists(xw))					// x-wave given
		imid = BinarySearchInterp(xw,0)
		imid = round(imid)
	else										// y-wave is assumed scaled
		imid = -DimOffset(yw,0)/DimDelta(yw,0)
	endif

	Duplicate/FREE yw, weight
	if (isPositiveInts(yw))						// assuming that yw is counts
		weight = yw[p]<1 ? 1 : 1/sqrt(yw[p])	// for counts, sigma=sqrt(yw)
	else
		weight = 1										// not counts, just set all weights to 1
	endif
	if (excludeHWE > 0)
		Variable minVal = min(WaveMin(yw),0), iLo,iHi
		if (WaveExists(xw))
			iLo = BinarySearch(xw,-excludeHWE)
			iHi = BinarySearch(xw,excludeHWE)
		else
			iLo = round(x2pnt(yw,-excludeHWE))
			iHi = round(x2pnt(yw,excludeHWE))
		endif
		weight[iLo,iHi] = minVal					// excluded region around elastic line
	endif

	if (WaveExists(fit_W))
		// fit_W = fitFunc(cw,fitX_W[p])
		CheckDisplayed/W=$gName fit_W
		if (!V_flag)
			AppendToGraph/W=$gName fit_W vs fitX_W
		endif
	endif

	String fitName=SelectString(useResolution,"DampedHarmonic","DampedHarmonicResWave")
	FUNCREF DampedHarmonic fitFunc = $fitName
	SetDataFolder fldr
	Variable err=0, V_FitError=0, V_FitQuitReason=0
	if (showStart)
		if (WaveExists(fit_W))
			FuncFit/Q/NTHR=0/O fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D=fit_W/NWOK
		else
			FuncFit/Q/NTHR=0/O fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D/NWOK
		endif
	else
		if (WaveExists(fit_W))
			FuncFit/Q/NTHR=0 fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D=fit_W/NWOK
		else
			FuncFit/Q/NTHR=0 fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D/NWOK
		endif
	endif
//	if (showStart)
//		if (WaveExists(xw))
//			FuncFit/Q/NTHR=0/O fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D=fit_W/NWOK
//		else
//			FuncFit/Q/NTHR=0/O fitFunc cw yw[pLo,pHi]/W=weight/D=fit_W/NWOK
//		endif
//	else
//		if (WaveExists(xw))
//			FuncFit/Q/NTHR=0 fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D=fit_W/NWOK
//		else
//			FuncFit/Q/NTHR=0 fitFunc cw yw[pLo,pHi]/W=weight/D=fit_W/NWOK
//		endif
//	endif
//	if (WaveExists(fit_W))
//		fit_W = fitFunc(cw,fitX_W[p])
//	endif

	//	FuncFit/NTHR=0 DampedHarmonic cw yw/X=xw/NWOK /D
	err = V_FitError || V_FitQuitReason
	for (i=0;i<Nph;i+=1)
		cw[2+3*i,3+3*i] = abs(cw[p])	// E(phonon) & gamma are always positive
	endfor

	Wave W_sigma=W_sigma
	if (V_FitError || V_FitQuitReason)
		print "Fit Failed ************************"
		print StringFromList(V_FitError,"","Any error;Singular matrix;Out of memory;Function returned NaN or INF;Fit function requested stop;Reentrant curve fitting;")
		if (V_FitQuitReason)
			print StringFromList(V_FitQuitReason,"","the iteration limit was reached;the user stopped the fit;the limit of passes without decreasing chi-square was reached;")
		endif
	endif
	SetDataFolder fldrSav0

	String str
	if (printIt && !err)
		if (showStart)
			Duplicate/FREE cw, sigma
			sigma = 0				// no error in the guess
		else
			Wave sigma = W_sigma
		endif
		printf "	Herix Eo	= %s meV,     chi^2 = %.5g\r",ValErrStr(cw[0],sigma[0],sp=1),V_chisq
		String tStr = SelectString(showStart,"Fit to peak(s):\r","Starting guess:\r")
		sprintf str, "%s\S2\M\t= %g\r", Letter2SymbolorUnicode("chi"), roundSignificant(V_chisq,3)
		tStr += str
		sprintf str, "E\\Bo\\M\t= %s\r", ValErrStr(cw[0],sigma[0],sp=1)
		tStr += str
		for (i=0;i<Nph;i+=1)
			printf " E(phonon) = %s meV,   gamma = %s meV,  amp = %s\r",ValErrStr(cw[2+i*3],sigma[2+i*3],sp=1),ValErrStr(cw[3+i*3],sigma[3+i*3],sp=1),ValErrStr(cw[1+i*3],sigma[1+i*3],sp=1)
			tStr += SelectString(i>=1,"","\r\r")
			sprintf str, "E\\Bph\\M\t= %s\r", ValErrStr(cw[2+i*3],sigma[2+i*3],sp=1)
			tStr += str
			sprintf str, "%s\t= %s\r", Letter2SymbolorUnicode("gamma"), ValErrStr(cw[3+i*3],sigma[3+i*3],sp=1)
			tStr += str
			sprintf str, "amp\t= %s", ValErrStr(cw[1+i*3],sigma[1+i*3],sp=1)
			tStr += str
		endfor
	endif

	String wnote="waveClass=DampedHarmonicCoefs;"
	if (WaveType(yw,2)==2)
		wnote = ReplaceStringByKey("sourceY",wnote,yList,"=")
		if (WaveExists(xw))
			wnote = ReplaceStringByKey("sourceX",wnote,xList,"=")
		endif
	else
		wnote = ReplaceStringByKey("sourceY",wnote,GetWavesDataFolder(yw,2),"=")
		if (WaveExists(xw))
			wnote = ReplaceStringByKey("sourceX",wnote,GetWavesDataFolder(xw,2),"=")
		endif
	endif
	if (useP)
		sprintf str,"[%g,%g]",pLo,pHi
		wnote = ReplaceStringByKey("subrange",wnote,str,"=")
	endif
	wnote = ReplaceStringByKey("function",wnote,fitName,"=")
	wnote = ReplaceStringByKey("startingCoefs",wnote,startCW,"=")
	if (WaveExists(resolutionWave))
		wnote = ReplaceStringByKey("resolutionWave",wnote,GetWavesDataFolder(resolutionWave,2),"=")
	endif
	if (!err)
		wnote = ReplaceNumberByKey("chisq",wnote,roundSignificant(V_chisq,5),"=")
		wnote = ReplaceStringByKey("W_sigma",wnote,vec2str(sigma,sep=","),"=")
	endif
	Note/K cw, wnote

	CheckDisplayed/W=$gName yw, fit_W
	if (V_flag && strlen(tStr))
#if (IgorVersion()<7)
		TextBox/C/N=textFitDamped/B=1/F=0/Y=3 tStr
#else
		TextBox/C/N=textFitDamped/B=(65535,65535,65535,50000)/F=0/Y=3 tStr
#endif
	endif
	return cw
End



Static Function/WAVE SetDampedHarmonicCoefsE(yw,xw,[Nph,cw,pLo,pHi,printIt])		// creates a first guess at the coefficients for a spectrum
	Wave yw,xw				// (x,y) data
	Variable Nph			// number of phonons (default is 1, yw has 2 peaks symmetric about 0 eV)
	Wave cw					// optional input wave to receive coeffs
	Variable pLo,pHi		// point range to fit
	Variable printIt		// forces printing

	Variable NphLast = NumVarOrDefault("root:Packages:DampedHarmonic:NphLast",1)
	Nph = ParamIsDefault(Nph) ? NaN : round(Nph)
	Nph = numtype(Nph) || Nph<1 ? NaN : Nph
	if (ParamIsDefault(cw))
		Wave cw = $""
	endif
	pLo = ParamIsDefault(pLo) ? -Inf : pLo
	pHi = ParamIsDefault(pHi) ? Inf : pHi
	printIt = ParamIsDefault(printIt) ? NaN : !(!printIt)
	printIt = numtype(printIt) ? strlen(GetRTStackInfo(2))<1 : printIt

	Variable medianWid=5, SVsmooth=21, phw=ceil(SVsmooth/2)

	Nph = (WaveExists(cw) && numtype(Nph)) ? floor((DimSize(cw,0)-1)/3) : Nph
	if (Nph<1 || numtype(Nph))
		Nph = numtype(Nph) || Nph<1 ? NphLast : Nph
		Prompt Nph,"Number of phonons to find", popup,"1 phonon pair;2 phonon pairs;3 phonon pairs;4 phonon pairs"
		DoPrompt "Number of Phonons",Nph
		if (V_flag)
			return $""
		endif
		printIt = 1
	endif
	if (Nph<1 || numtype(Nph))
		return $""
	endif

	String fitInfo=""
	if (!WaveExists(yw))
		fitInfo = WhatToFitInfoMulti("",pLo=pLo,pHi=pHi)
		String fldr=StringByKey("fldr",fitInfo,"=")
		if (!WaveExists(yw))
			Wave yw = $(fldr+StringByKey("yWave",fitInfo,"="))
		endif
		if (!WaveExists(xw))
		Wave xw = $(fldr+StringByKey("xWave",fitInfo,"="))
		endif
		pLo = NumberByKey("pLo",fitInfo,"=")
		pHi = NumberByKey("pHi",fitInfo,"=")
		if (!WaveExists(yw) || numtype(pLo+pHi))
			print "ERROR -- SetDampedHarmonicCoefsE(), Cannot find data waves from top graph"
			return $""
		elseif (NumberByKey("xLo",fitInfo,"=")*NumberByKey("xHi",fitInfo,"=") >= 0)
			print "ERROR -- SetDampedHarmonicCoefsE(), selected range does not span origin"
			return $""
		endif
	endif
	Variable T_K=T_SampleFromYwave(yw,T_K_default)

	if (!WaveExists(cw))
		Make/N=(1+3*Nph)/O/D Coefs		// Eo and 3 numbers for each phonon
		Wave cw = Coefs
	else
		Redimension/N=(1+3*Nph)/D cw
	endif
	SetDimLabel 0, 0, Eo, cw
	String str
	Variable i
	for (i=0;i<Nph;i+=1)
		str = "amp"+num2istr(i+1)		;	SetDimLabel 0, 1+i*3, $str, cw
		str = "phonon"+num2istr(i+1)	;	SetDimLabel 0, 2+i*3, $str, cw
		str = "gamma"+num2istr(i+1)	;	SetDimLabel 0, 3+i*3, $str, cw
	endfor
	Init_DampedHarmonic()
	Variable/G root:Packages:DampedHarmonic:NphLast = Nph
	Variable excludeHWE = NumVarOrDefault("root:Packages:DampedHarmonic:excludeHWE",excludeHWdefaultE)

	Variable HerixEo = FindCenterOfSpectrumE(yw,xw,NaN)
	Duplicate/FREE yw, ywS				// make a smoothed version of yw
	if (excludeHWE > 0)
		Variable minVal = min(WaveMin(ywS),0), iLo,iHi
		if (WaveExists(xw))
			iLo = BinarySearch(xw,-excludeHWE)
			iHi = BinarySearch(xw,excludeHWE)
		else
			iLo = round(x2pnt(yw,-excludeHWE))
			iHi = round(x2pnt(yw,excludeHWE))
		endif
		ywS[iLo,iHi] = minVal
	endif
	Smooth/M=0 medianWid, ywS			// first a median filter to remove outliers
	Smooth/S=2 SVsmooth, ywS			// then a Savitzky-Golay, to smooth peaks

	Variable phonon, gam, amp,   B_E, val, maxLo, maxHi
	Variable EofMax, pMax, N=DimSize(yw,0)
	cw[0] = HerixEo==0 ? 0.02 : HerixEo
	for (i=0;i<Nph;i+=1)
		WaveStats/M=1/Q/Z ywS
		maxLo = V_max
		pMax = V_maxloc
		EofMax = WaveExists(xw) ? xw[pMax] : DimOffset(yw,0)+pMax*DimDelta(yw,0)
		ywS[max(pMax-phw,0),min(pMax+phw,N-1)] = minVal	// zero out the peak just identified

		EofMax *= -1													// energy of associated peak for this phonon
		pMax = WaveExists(xw) ? BinarySearch(xw,EofMax) : (EofMax-DimOffset(yw,0)) / DimDelta(yw,0)
		ywS[max(pMax-phw,0),min(pMax+phw,N-1)] = minVal	// zero the mating peak

		phonon = abs(EofMax)
		gam = phonon/5

		// calcualte guess at amplitude
		B_E = abs(1 + 1/(exp(-phonon/(kB*T_K))-1))		// Bose-Einstein occupation {1 + n(-phonon)}
		val = B_E / abs(gam*phonon)
		maxLo = maxLo / val
		B_E = abs(1 + 1/(exp(abs(phonon)/(kB*T_K))-1))// Bose-Einstein occupation {1 + n(phonon)}
		val = B_E / abs(gam*phonon)
		maxHi = maxHi / val
		amp = (maxLo+maxHi)/2

		cw[3*i+1] = amp
		cw[3*i+2] = phonon
		cw[3*i+3] = gam
	endfor

	String wnote=note(cw)
	wnote = MergeKeywordLists(wnote,fitInfo,1,"=","")
	Note/K cw, ReplaceStringByKey("waveClass",wnote,"DampedHarmonicCoefs","=")

	if (printIt)
		printf "Created coefficients wave  \"%s\" = %s,  %d phonons\r",NameOfWave(cw),vec2str(cw),Nph
	endif
	return cw
End


Static Function/T WhatToFitInfoMulti(gName,[pLo,pHi,xLo,xHi])
	String gName			// graph name, "" means top graph
	Variable pLo,pHi		// point range to fit (optional)
	Variable xLo, xHi
	pLo = ParamIsDefault(pLo) || numtype(pLo) ? -Inf : pLo
	pHi = ParamIsDefault(pHi) || numtype(pHi) ? Inf : pHi
	xLo = ParamIsDefault(xLo) || numtype(xLo) ? -Inf : xLo
	xHi = ParamIsDefault(xHi) || numtype(xHi) ? Inf : xHi
	if (numtype(pLo+xLo)==0 || numtype(pHi+xHi)==0)
		print "ERROR -- WhatToFitInfoMulti(), cannot specify both point range and x range on input"
		return ""
	endif

	String glist = WinList("*",";","WIN:1")	// list of all graph windows
	if (strlen(gName)<1)
		gname = StringFromList(0,glist)
	endif
	if (WhichListItem(gName,glist)<0 || strlen(gname)<1)
		return ""										// graph does not exist
	endif

	String traceNames=TraceNameList(gName,";",1)
	String name0=StringFromList(0,traceNames)
	String yList="", xList="", tname
	Variable i
	for (i=0;i<ItemsInList(traceNames);i+=1)
		tname = StringFromList(i,traceNames)
		if (strsearch(tname,name0,0)==0)
			yList += GetWavesDataFolder(TraceNameToWaveRef(gName,tname),2)+";"
			Wave wx = XWaveRefFromTrace(gName,tname)// xwave if one exists
			xList += GetWavesDataFolder(wx,2)+";"
		endif
	endfor

	tname = StringFromList(0,traceNames)
	Wave wy = TraceNameToWaveRef(gName,tname)
	Wave wx = XWaveRefFromTrace(gname,tname)	// xwave if one exists
	if (!WaveExists(wy))
		return ""
	endif

	if (ItemsInList(yList)>1 && (numtype(pLo)==0 || numtype(pHi)==0))	// many y waves, but pLo or pHi given
		print "ERROR -- WhatToFitInfoMulti(), Cannot give a point number for fit range when using multiple curves"
	endif

	Variable iLast

	String infoA=CsrInfo(A,gName), infoB=CsrInfo(B,gName)
	// find the xHi and pHi
	if (strlen(infoA) && numtype(xLo) && numtype(pLo))		// neither xLo nor pLo given, but infoA is OK
		xLo = hcsr(A,gName)
		pLo = pcsr(A,gName)
	elseif (ItemsInList(yList)==1 && numtype(pLo)==0)		// only ONE y wave and pLo given
		Wave wy = $StringFromList(0,yList)
		Wave wx = $StringFromList(0,xList)
		if (WaveExists(wx))
			iLast = min(DimSize(wx,0)-1,DimSize(wy,0)-1)
			pLo = limit(pLo,0,iLast)
			xLo = wx[pLo]
		else
			pLo = limit(pLo,0,DimSize(wy,0)-1)
			xLo = pLo*DimDelta(wy,0) + DimOffset(wy,0)
		endif
	elseif (numtype(xLo))												// xLo not given, find the lowest x value
		xLo = -Inf
		pLo = -Inf
	endif

	// find the xHi and pHi
	if (strlen(infoB) && numtype(xHi) && numtype(pHi))		// neither xHi nor pHi given, but infoB is OK
		xHi = hcsr(B,gName)
		pHi = pcsr(B,gName)
	elseif (ItemsInList(yList)==1 && numtype(pHi)==0)		// only ONE y wave and pHi given
		Wave wy = $StringFromList(0,yList)
		Wave wx = $StringFromList(0,xList)
		if (WaveExists(wx))
			iLast = min(DimSize(wx,0)-1,DimSize(wy,0)-1)
			pHi = limit(pHi,0,iLast)
			xHi = wx[pHi]
		else
			pHi = limit(pHi,0,DimSize(wy,0)-1)
			xHi = pHi*DimDelta(wy,0) + DimOffset(wy,0)
		endif
	elseif (numtype(xHi))												// xHi not given, find the highest x value
		xHi = Inf
		pHi = Inf
	endif

	if (xLo>xHi)															// swap if necessary
		Variable pswap=pLo, xswap=xLo
		pLo = pHi
		pHi = pswap
		xLo = xHi
		xHi = xswap
	endif

	String list = ReplaceStringByKey("fldr","",GetWavesDataFolder(wy,1),"=")
	list = ReplaceStringByKey("graphName",list,gName,"=")
	if (ItemsInList(yList)==1)
		Wave wy = $StringFromList(0,ylist)
		Wave wx = $StringFromList(0,xlist)
		list = ReplaceStringByKey("yWave",list,NameOfWave(wy),"=")
		if (WaveExists(wx))
			list = ReplaceStringByKey("xWave",list,NameOfWave(wx),"=")
		endif
		list = ReplaceNumberByKey("pLo",list,pLo,"=")
		list = ReplaceNumberByKey("pHi",list,pHi,"=")
	endif
	yList = ReplaceString(";",ylist,",")
	xList = ReplaceString(";",xlist,",")
	list = ReplaceStringByKey("yWaveList",list,yList,"=")
	list = ReplaceStringByKey("xWaveList",list,xList,"=")
	list = ReplaceNumberByKey("xLo",list,xLo,"=")
	list = ReplaceNumberByKey("xHi",list,xHi,"=")
	list = ReplaceNumberByKey("useCursors",list,strlen(infoA)>1,"=")
	return list
End

Function FindCenterOfSpectrumE(yw,xw,hw,[printIt])
	Wave yw, xw
	Variable hw								// half-width in energy, use NaN for auto-choose
	Variable printIt		// forces printing
	printIt = ParamIsDefault(printIt) ? NaN : !(!printIt)
	printIt = numtype(printIt) ? strlen(GetRTStackInfo(2))<1 : printIt
	if (!WaveExists(yw))
		return NaN
	endif

	Variable xLo,xHi
	if (WaveExists(xw))					// x-wave given
		WaveStats/M=1/Q xw
		xLo = V_min
		xHi = V_max
	else										// y-wave is assumed scaled
		xLo = pnt2x(yw,0)
		xHi = pnt2x(yw,DimSize(yw,0)-1)
	endif

	Variable xEnd = min(abs(xLo),abs(xHi))
	//print "xEnd = ",xEnd
	hw = numtype(hw) || hw<=0 ? 0.1*xEnd : hw
	//print "hw = ",hw
	xEnd -= hw
	if (hw < 1 || !((xLo*xHi)<0) || xEnd<=0)	// least 1 meV, must span zero, and hw<xEnd
		return NaN
	endif

	Variable dE=xEnd/50					// use a resolution of 1/50 of total HW of data
	dE = limit(dE,0.05,0.2)			// force dE to range [0.05, 0.1]
	Variable N=floor(xEnd/dE)
	Variable NcHW=floor(hw/dE), Nc=2*NcHW + 1
	Make/N=(N)/D/FREE corrPlus, corrNeg
	Make/N=(Nc)/D/FREE corr=0
	SetScale/P x, 0, dE, corrPlus, corrNeg

	Variable ic, Ec
	for (ic=-NcHW; ic<=NcHW; ic+=1)
		Ec = ic*dE
		if (WaveExists(xw))
			corrPlus = interp(x+Ec,xw,yw)
			corrNeg = interp(-x+Ec,xw,yw)
		else
			corrPlus = yw(x+Ec)
			corrNeg = yw(-x+Ec)
		endif
		//Duplicate/O corrPlus, corrPlusView
		//Duplicate/O corrNeg, corrNegView
		MatrixOp/FREE cc = sum(corrPlus * corrNeg)
		corr[ic+NcHW] = cc[0]
	endfor
	WaveStats/Q/M=1 corr
	Ec = (V_maxloc-NcHW)*dE
	//Duplicate/O corr, corrView

	if (printIt)
		printf "FindCenterOfSpectrumE(%s, %s, %g",NameOfWave(yw),NameOfWave(xw),hw
		if (!ParamIsDefault(printIt))
			printf ", printIt=%g", printIt
		endif
		printf ") // E(meV) = %g\r",Ec
		if (WaveType(yw,2)==2)
			printf "    y-waves = %s\r",StringByKey("yWaveList",note(yw),"=")
			if (strlen(StringByKey("yWaveList",note(yw),"=")))
				printf "    x-waves = %s\r",StringByKey("xWaveList",note(yw),"=")
			endif
		endif
	endif
	return Ec
End

//  =============================== End of Multi-Scan Fit ================================  //
//  ======================================================================================  //




//  ======================================================================================  //
//  ============================== Start of Single-Scan Fit ==============================  //

Function SetCentralExclusionHWp(hw,[printIt])
	Variable hw
	Variable printIt
	printIt = ParamIsDefault(printIt) ? NaN : printIt
	printIt = numtype(printIt)  ? strlen(GetRTStackInfo(2))<1 : !(!printIt)

	Variable hwDefault = NumVarOrDefault("root:Packages:DampedHarmonic:excludeHW",excludeHWdefault)
	if (hw<0 || numtype(hw))								// bad value, ask user
		hw = hwDefault
		Prompt hw,"Exclude  points about Zero"
		DoPrompt "Exclude HW (points)", hw
		if (V_flag)
			return hwDefault
		endif
		printIt = 1
	endif
	if (printIt)
		printf "SetCentralExclusionHWp(%g)\r",hw
	endif

	if (hw>=0 && numtype(hw)==0)							// valid input
		Init_DampedHarmonic()													// ensure data folders exists
		Variable/G root:Packages:DampedHarmonic:excludeHW = hw		// good value, set it
	else
		hw = hwDefault
	endif
	return hw
End


Function/WAVE FitDampedHarmonic(yw,xw,[Nph,cw,useResolution,pLo,pHi,showStart,printIt])
	Wave yw,xw				// (x,y) data
	Variable Nph			// number of phonons (default is 1, yw has 2 peaks symmetric about 0 eV)
	Wave cw					// optional input wave to receive coeffs
	Variable useResolution
	Variable pLo,pHi		// point range to fit
	Variable showStart	// only show starting guess, do not fit
	Variable printIt		// forces printing

	Variable NphLast = NumVarOrDefault("root:Packages:DampedHarmonic:NphLast",1)
	Nph = ParamIsDefault(Nph) ? NaN : round(Nph)
	Nph = numtype(Nph) || Nph<1 ? NaN : Nph
	if (ParamIsDefault(cw))
		Wave cw = $""
	endif
	useResolution = ParamIsDefault(useResolution) ? NaN : !(!useResolution)
	useResolution = numtype(useResolution) ? NaN : useResolution
	pLo = ParamIsDefault(pLo) ? -Inf : pLo
	pHi = ParamIsDefault(pHi) ? Inf : pHi
	showStart = ParamIsDefault(showStart) ? 0 : showStart
	showStart = numtype(showStart) ? 0 : !(!showStart)
	printIt = ParamIsDefault(printIt) ? NaN : !(!printIt)
	printIt = numtype(printIt) ? strlen(GetRTStackInfo(2))<1 : printIt

	Nph = (WaveExists(cw) && numtype(Nph)) ? floor((DimSize(cw,0)-1)/3) : Nph
	if (Nph<1 || numtype(Nph) || numtype(useResolution))
		useResolution = useResolution || numtype(useResolution) ? 1 : 2
		Nph = numtype(Nph) || Nph<1 ? NphLast : Nph
		Prompt useResolution,"Use Measured Resolution", popup,"Use Resolution;Do NOT use Resolution"
		Prompt Nph,"Number of phonons to find", popup,"1 phonon pair;2 phonon pairs;3 phonon pairs;4 phonon pairs"
		Prompt showStart, "Fit or just show starting guess", popup, "Fit;Starting Guess Only"
		showStart = showStart ? 2 : 1
		DoPrompt "Number of Phonons",Nph, useResolution, showStart
		if (V_flag)
			return $""
		endif
		useResolution = useResolution==1
		showStart = showStart==2
		printIt = 1
	endif
	if (Nph<1 || numtype(Nph) || numtype(useResolution))
		return $""
	endif

	String fitInfo = VoigtDeconvolve#WhatToFitInfo("",pLo=pLo,pHi=pHi)
	String fldr=StringByKey("fldr",fitInfo,"=")
	if (!WaveExists(yw))
		Wave yw = $(fldr+StringByKey("yWave",fitInfo,"="))
	endif
	if (!WaveExists(xw))
		Wave xw = $(fldr+StringByKey("xWave",fitInfo,"="))
	endif
	pLo = NumberByKey("pLo",fitInfo,"=")
	pHi = NumberByKey("pHi",fitInfo,"=")
	Variable xLo=NumberByKey("xLo",fitInfo,"="), xHi=NumberByKey("xHi",fitInfo,"=")

	if (!WaveExists(yw) || numtype(pLo+pHi))
		print "ERROR -- FitDampedHarmonic(), Cannot find data waves from top graph"
		return $""
	elseif (xHi*xLo >= 0)
		print "ERROR -- FitDampedHarmonic(), selected range does not span origin"
		return $""
	endif
	String fldrSav0= GetDataFolder(1)

	Variable T_K=T_SampleFromYwave(yw,T_K_default)
	String units = SelectString(WaveExists(xw),WaveUnits(yw,0),WaveUnits(xw,-1))
	Variable useP = pLo>0 || pHi<DimSize(yw,0)-1

	if (printIt)
		String xname = SelectString(WaveExists(xw),"$\"\"",GetWavesDataFolder(xw,2))
		printf "FitDampedHarmonic(%s,%s",GetWavesDataFolder(yw,2),xname
		if (!ParamIsDefault(cw))
			printf ", cw=%s",GetWavesDataFolder(cw,2)
		endif
		if (!ParamIsDefault(useResolution) || useResolution)
			printf ", useResolution=%g",useResolution
		endif
		if (useP)
			printf ", pLo=%g,pHi=%g",pLo,pHi
		endif
		printf ")\r"
	endif
	if (!WaveExists(cw))
		SetDataFolder fldr
		Wave cw = SetDampedHarmonicCoefs(yw,xw,Nph=Nph)
		SetDataFolder fldrSav0
	endif
	if (!WaveExists(cw))
		print "ERROR -- FitDampedHarmonic(), Could not get the Coefficients wave"
		return $""
	endif
	Variable excludeHW = NumVarOrDefault("root:Packages:DampedHarmonic:excludeHW",excludeHWdefault)
	if (printIt)
		printf "	Starting Fit with coefficients \"%s\" = %s\r",GetWavesDataFolder(cw,2),vec2str(cw)
		printf "    and using T = %g K,   Excluded HW = %g points\r",T_K, excludeHW
	endif
	String startCW=vec2str(cw,sep=",")

	Wave resolutionWave=root:resolutionWave				// ensure that the resolution wave exists
	if (!WaveExists(resolutionWave))
		Wave resolutionWave=herix#SetResolutionWave(yw)
	endif
	if (!WaveExists(resolutionWave))
		DoAlert 0, "Unable to get resolution wave"
		return $""
	endif
	Init_DampedHarmonic()
	Variable/G root:Packages:DampedHarmonic:NphLast = Nph

	Variable imid
	if (WaveExists(xw))					// x-wave given
		imid = BinarySearch(xw,0)
	else										// y-wave is assumed scaled
		imid = -DimOffset(yw,0)/DimDelta(yw,0)
	endif

	Duplicate/FREE yw, weight
	if (isPositiveInts(yw))						// assuming that yw is counts
		weight = yw[p]<1 ? 1 : 1/sqrt(yw[p])	// for counts, sigma=sqrt(yw)
	else
		weight = 1										// not counts, just set all weights to 1
	endif
	weight[imid-excludeHW,imid+excludeHW] = 0	// excluded region around elastic line

	String fitName=SelectString(useResolution,"DampedHarmonic","DampedHarmonicResWave")
	FUNCREF DampedHarmonic fitFunc = $fitName

	SetDataFolder fldr
	Variable i, err=0, V_FitError=0, V_FitQuitReason=0

	if (showStart)
		if (WaveExists(xw))
			FuncFit/Q/NTHR=0/O fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D
		else
			FuncFit/Q/NTHR=0/O fitFunc cw yw[pLo,pHi]/W=weight/D
		endif
	else
		if (WaveExists(xw))
			FuncFit/Q/NTHR=0 fitFunc cw yw[pLo,pHi]/X=xw/W=weight/D
		else
			FuncFit/Q/NTHR=0 fitFunc cw yw[pLo,pHi]/W=weight/D
		endif
	endif
	//	FuncFit/NTHR=0 DampedHarmonic cw yw/X=xw/NWOK /D
	err = V_FitError || V_FitQuitReason
	for (i=0;i<Nph;i+=1)
		cw[2+3*i,3+3*i] = abs(cw[p])	// E(phonon) & gamma are always positive
	endfor

	Wave W_sigma=W_sigma
	if (V_FitError || V_FitQuitReason)
		print "Fit Failed ************************"
		print StringFromList(V_FitError,"","Any error;Singular matrix;Out of memory;Function returned NaN or INF;Fit function requested stop;Reentrant curve fitting;")
		if (V_FitQuitReason)
			print StringFromList(V_FitQuitReason,"","the iteration limit was reached;the user stopped the fit;the limit of passes without decreasing chi-square was reached;")
		endif
	endif
	SetDataFolder fldrSav0

	String str
	if (printIt && !err)
		if (showStart)
			Duplicate/FREE cw, sigma
			sigma = 0				// no error in the guess
		else
			Wave sigma = W_sigma
		endif
		printf "	Herix Eo	= %s meV,     chi^2 = %.5g\r",ValErrStr(cw[0],sigma[0],sp=1),V_chisq
		String tStr = SelectString(showStart,"Fit to peak(s):\r","Starting guess:\r")
		sprintf str, "%s\S2\M\t= %g\r", Letter2SymbolorUnicode("chi"), roundSignificant(V_chisq,3)
		tStr += str
		sprintf str, "E\\Bo\\M\t= %s\r", ValErrStr(cw[0],sigma[0],sp=1)
		tStr += str
		for (i=0;i<Nph;i+=1)
			printf " E(phonon) = %s meV,   gamma = %s meV,  amp = %s\r",ValErrStr(cw[2+i*3],sigma[2+i*3],sp=1),ValErrStr(cw[3+i*3],sigma[3+i*3],sp=1),ValErrStr(cw[1+i*3],sigma[1+i*3],sp=1)
			tStr += SelectString(i>=1,"","\r\r")
			sprintf str, "E\\Bph\\M\t= %s\r", ValErrStr(cw[2+i*3],sigma[2+i*3],sp=1)
			tStr += str
			sprintf str, "%s\t= %s\r", Letter2SymbolorUnicode("gamma"), ValErrStr(cw[3+i*3],sigma[3+i*3],sp=1)
			tStr += str
			sprintf str, "amp\t= %s", ValErrStr(cw[1+i*3],sigma[1+i*3],sp=1)
			tStr += str
		endfor
	endif

	String wnote="waveClass=DampedHarmonicCoefs;"
	wnote = ReplaceStringByKey("sourceY",wnote,GetWavesDataFolder(yw,2),"=")
	if (WaveExists(xw))
		wnote = ReplaceStringByKey("sourceX",wnote,GetWavesDataFolder(xw,2),"=")
	endif
	if (useP)
		sprintf str,"[%g,%g]",pLo,pHi
		wnote = ReplaceStringByKey("subrange",wnote,str,"=")
	endif
	wnote = ReplaceStringByKey("function",wnote,fitName,"=")
	wnote = ReplaceStringByKey("startingCoefs",wnote,startCW,"=")
	if (WaveExists(resolutionWave))
		wnote = ReplaceStringByKey("resolutionWave",wnote,GetWavesDataFolder(resolutionWave,2),"=")
	endif
	if (!err)
		wnote = ReplaceNumberByKey("chisq",wnote,roundSignificant(V_chisq,5),"=")
		wnote = ReplaceStringByKey("W_sigma",wnote,vec2str(sigma,sep=","),"=")
	endif
	Note/K cw, wnote

	String gName = StringFromList(0,WinList("*",";","WIN:1"))
	CheckDisplayed/W=$gName yw
	if (V_flag && strlen(tStr))
#if (IgorVersion()<7)
		TextBox/C/N=textFitDamped/B=1/F=0/Y=3 tStr
#else
		TextBox/C/N=textFitDamped/B=(65535,65535,65535,50000)/F=0/Y=3 tStr
#endif
	endif

	return cw
End



Function/WAVE SetDampedHarmonicCoefs(yw,xw,[Nph,cw,pLo,pHi,printIt])		// creates a first guess at the coefficients for a spectrum
	Wave yw,xw				// (x,y) data
	Variable Nph			// number of phonons (default is 1, yw has 2 peaks symmetric about 0 eV)
	Wave cw					// optional input wave to receive coeffs
	Variable pLo,pHi		// point range to fit
	Variable printIt		// forces printing

	Variable NphLast = NumVarOrDefault("root:Packages:DampedHarmonic:NphLast",1)
	Nph = ParamIsDefault(Nph) ? NaN : round(Nph)
	Nph = numtype(Nph) || Nph<1 ? NaN : Nph
	if (ParamIsDefault(cw))
		Wave cw = $""
	endif
	pLo = ParamIsDefault(pLo) ? -Inf : pLo
	pHi = ParamIsDefault(pHi) ? Inf : pHi
	printIt = ParamIsDefault(printIt) ? NaN : !(!printIt)
	printIt = numtype(printIt) ? strlen(GetRTStackInfo(2))<1 : printIt

	Variable medianWid=5, SVsmooth=21, phw=ceil(SVsmooth/2)

	Nph = (WaveExists(cw) && numtype(Nph)) ? floor((DimSize(cw,0)-1)/3) : Nph
	if (Nph<1 || numtype(Nph))
		Nph = numtype(Nph) || Nph<1 ? NphLast : Nph
		Prompt Nph,"Number of phonons to find", popup,"1 phonon pair;2 phonon pairs;3 phonon pairs;4 phonon pairs"
		DoPrompt "Number of Phonons",Nph
		if (V_flag)
			return $""
		endif
		printIt = 1
	endif
	if (Nph<1 || numtype(Nph))
		return $""
	endif

	if (Nph==1)								//when only 1 phonon, use the old method
		return SetDampedHarmonicCoefsOnly1(yw,xw,cw=cw,pLo=pLo,pHi=pHi,printIt=printIt)
		Init_DampedHarmonic()
		Variable/G root:Packages:DampedHarmonic:NphLast = Nph
	endif

	String fitInfo = VoigtDeconvolve#WhatToFitInfo("",pLo=pLo,pHi=pHi)
	String fldr=StringByKey("fldr",fitInfo,"=")
	if (!WaveExists(yw))
		Wave yw = $(fldr+StringByKey("yWave",fitInfo,"="))
	endif
	if (!WaveExists(xw))
		Wave xw = $(fldr+StringByKey("xWave",fitInfo,"="))
	endif
	pLo = NumberByKey("pLo",fitInfo,"=")
	pHi = NumberByKey("pHi",fitInfo,"=")
	if (!WaveExists(yw) || numtype(pLo+pHi))
		print "ERROR -- SetDampedHarmonicCoefs(), Cannot find data waves from top graph"
		return $""
	elseif (NumberByKey("xLo",fitInfo,"=")*NumberByKey("xHi",fitInfo,"=") >= 0)
		print "ERROR -- SetDampedHarmonicCoefs(), selected range does not span origin"
		return $""
	endif
	if (!WaveExists(cw))
		Make/N=(1+3*Nph)/O/D Coefs		// Eo and 3 numbers for each phonon
		Wave cw = Coefs
	else
		Redimension/N=(1+3*Nph)/D cw
	endif
	SetDimLabel 0, 0, Eo, cw
	String str
	Variable i
	for (i=0;i<Nph;i+=1)
		str = "amp"+num2istr(i+1)		;	SetDimLabel 0, 1+i*3, $str, cw
		str = "phonon"+num2istr(i+1)	;	SetDimLabel 0, 2+i*3, $str, cw
		str = "gamma"+num2istr(i+1)	;	SetDimLabel 0, 3+i*3, $str, cw
	endfor
	Init_DampedHarmonic()
	Variable/G root:Packages:DampedHarmonic:NphLast = Nph
	Variable excludeHW = NumVarOrDefault("root:Packages:DampedHarmonic:excludeHW",excludeHWdefault)

	Variable HerixEo=0
	Variable imid = FindCenterOfSpectrum(yw,xw,max(excludeHW,5))	// point number of E=0
	if (numtype(imid))					// correlation method failed, fall back to dead reckoning
		if (WaveExists(xw))				// x-wave given
			imid = BinarySearch(xw,0)
		else									// y-wave is assumed scaled
			imid = -DimOffset(yw,0)/DimDelta(yw,0)
		endif
	else
		if (WaveExists(xw))				// x-wave given, get the x-value
			HerixEo = xw[imid]
		else									// y-wave is assumed scaled
			HerixEo = imid*DimDelta(yw,0) + DimOffset(yw,0)
		endif
	endif

	Duplicate/FREE yw, ywS				// make a smoothed version of yw
	if (excludeHW > 0)
		Variable minVal = min(WaveMin(ywS),0)
		ywS[imid-excludeHW,imid+excludeHW] = minVal
	endif
	Smooth/M=0 medianWid, ywS			// first a median filter to remove outliers
	Smooth/S=2 SVsmooth, ywS			// then a Savitzky-Golay, to smooth peaks

	Variable T_K=T_SampleFromYwave(yw,T_K_default)
	Variable phonon, gam, amp,   B_E, val, maxLo, maxHi
	Variable EofMax, pMax, N=DimSize(yw,0)
	cw[0] = HerixEo==0 ? 0.02 : HerixEo
	for (i=0;i<Nph;i+=1)
		WaveStats/M=1/Q/Z ywS
		maxLo = V_max
		pMax = V_maxloc
		EofMax = WaveExists(xw) ? xw[pMax] : DimOffset(yw,0)+pMax*DimDelta(yw,0)
		ywS[max(pMax-phw,0),min(pMax+phw,N-1)] = minVal	// zero out the peak just identified

		EofMax *= -1													// energy of associated peak for this phonon
		pMax = WaveExists(xw) ? BinarySearch(xw,EofMax) : (EofMax-DimOffset(yw,0)) / DimDelta(yw,0)
		ywS[max(pMax-phw,0),min(pMax+phw,N-1)] = minVal	// zero the mating peak

		phonon = abs(EofMax)
		gam = phonon/5

		// calcualte guess at amplitude
		B_E = abs(1 + 1/(exp(-phonon/(kB*T_K))-1))		// Bose-Einstein occupation {1 + n(-phonon)}
		val = B_E / abs(gam*phonon)
		maxLo = maxLo / val
		B_E = abs(1 + 1/(exp(abs(phonon)/(kB*T_K))-1))// Bose-Einstein occupation {1 + n(phonon)}
		val = B_E / abs(gam*phonon)
		maxHi = maxHi / val
		amp = (maxLo+maxHi)/2

		cw[3*i+1] = amp
		cw[3*i+2] = phonon
		cw[3*i+3] = gam
	endfor

	String wnote = ReplaceStringByKey("waveClass",note(cw),"DampedHarmonicCoefs","=")
	wnote = ReplaceNumberByKey("T_Sample",wnote,T_K,"=")
	Note/K cw, wnote

	if (printIt)
		printf "Created coefficients wave  \"%s\" = %s,  %d phonons\r",NameOfWave(cw),vec2str(cw),Nph
	endif
	return cw
End
//
//	This is the old way.  It is better when only 1 phonon
Static Function/WAVE SetDampedHarmonicCoefsOnly1(yw,xw,[cw,pLo,pHi,printIt])		// creates a first guess at the coefficients for a spectrum
	Wave yw,xw				// (x,y) data
	Wave cw					// optional input wave to receive coeffs
	Variable pLo,pHi		// point range to fit
	Variable printIt		// forces printing
	if (ParamIsDefault(cw))
		Wave cw = $""
	endif
	pLo = ParamIsDefault(pLo) ? -Inf : pLo
	pHi = ParamIsDefault(pHi) ? Inf : pHi
	printIt = ParamIsDefault(printIt) ? NaN : !(!printIt)
	printIt = numtype(printIt) ? strlen(GetRTStackInfo(2))<1 : printIt

	String fitInfo = VoigtDeconvolve#WhatToFitInfo("",pLo=pLo,pHi=pHi)
	String fldr=StringByKey("fldr",fitInfo,"=")
	if (!WaveExists(yw))
		Wave yw = $(fldr+StringByKey("yWave",fitInfo,"="))
	endif
	if (!WaveExists(xw))
		Wave xw = $(fldr+StringByKey("xWave",fitInfo,"="))
	endif
	pLo = NumberByKey("pLo",fitInfo,"=")
	pHi = NumberByKey("pHi",fitInfo,"=")
	if (!WaveExists(yw) || numtype(pLo+pHi))
		print "ERROR -- SetDampedHarmonicCoefsOnly1(), Cannot find data waves from top graph"
		return $""
	elseif (NumberByKey("xLo",fitInfo,"=")*NumberByKey("xHi",fitInfo,"=") >= 0)
		print "ERROR -- SetDampedHarmonicCoefsOnly1(), selected range does not span origin"
		return $""
	endif
	if (!WaveExists(cw))
		Make/N=4/O/D Coefs
		Wave cw = Coefs
	else
		Redimension/N=4/D cw
	endif
	SetDimLabel 0, 0, Eo, cw
	SetDimLabel 0, 1, amp, cw
	SetDimLabel 0, 2, phonon, cw
	SetDimLabel 0, 3, gamma, cw

	Variable imid
	if (WaveExists(xw))					// x-wave given
		imid = BinarySearch(xw,0)
	else										// y-wave is assumed scaled
		imid = -DimOffset(yw,0)/DimDelta(yw,0)
	endif
	Variable excludeHW = NumVarOrDefault("root:Packages:DampedHarmonic:excludeHW",excludeHWdefault)
	WaveStats/M=1/Q/R=[0,imid-excludeHW]/Z yw
	Variable xloMax=V_maxloc, maxLo=V_max
	WaveStats/M=1/Q/R=[imid+excludeHW,numpnts(yw)-1]/Z yw
	Variable xhiMax=V_maxloc, maxHi=V_max
	if (WaveExists(xw))					// x-wave given
		xloMax = xw[xloMax]
		xhiMax = xw[xhiMax]
	endif

	Variable phonon = (xhiMax-xloMax)/2
	Variable HerixEo = (xhiMax+xloMax)/2
	Variable gam = phonon/5
	Variable T_K=T_SampleFromYwave(yw,T_K_default)

	// calcualte guess at amplitude
	Variable B_E = abs(1 + 1/(exp(xloMax/(kB*T_K))-1))	// Bose-Einstein occupation {1 + n(xloMax)}
	Variable val = B_E * abs(gam*xloMax) / ( (xloMax*xloMax-phonon*phonon)^2 + (gam*xloMax)^2 )
	maxLo = maxLo / val
	B_E = abs(1 + 1/(exp(xhiMax/(kB*T_K))-1))	// Bose-Einstein occupation {1 + n(xhiMax)}
	val = B_E * abs(gam*xhiMax) / ( (xhiMax*xhiMax-phonon*phonon)^2 + (gam*xhiMax)^2 )
	maxHi = maxHi / val
	Variable amp = (maxLo+maxHi)/2

	cw = {HerixEo, amp, phonon, gam}
	String wnote = ReplaceStringByKey("waveClass",note(cw),"DampedHarmonicCoefs","=")
	wnote = ReplaceNumberByKey("T_Sample",wnote,T_K,"=")
	Note/K cw, wnote

	if (printIt)
		printf "Created coefficients wave  \"%s\" = %s\r",NameOfWave(cw),vec2str(cw)
	endif
	return cw
End

//  =============================== End of Single-Scan Fit ===============================  //
//  ======================================================================================  //




//  ======================================================================================  //
//  ============================= Start of Common Functions ==============================  //

Function/S SetTemperaturePVorWaveName(Tname)
	String Tname						// name of PV or wave that holds the Temperature

	Init_DampedHarmonic()												// ensure data folders exists
	if (strlen(Tname)>0)
		String/G root:Packages:DampedHarmonic:Tname = Tname	// good value passed, set it
		return Tname
	endif

	Tname = StrVarOrDefault("root:Packages:DampedHarmonic:Tname",Temperature_PVorWave)
	Prompt Tname,"Name of PV or wave with Sample Temperature (K)"
	DoPrompt "Sample Temperature", Tname
	if (V_flag)
		return Tname
	endif
	if (strlen(Tname)>0)
		String/G root:Packages:DampedHarmonic:Tname = Tname	// good value entered, set it
	endif
	return Tname
End


Function OverRideSampleTemperature(T_K)
	Variable T_K

	Init_DampedHarmonic()												// ensure data folders exists
	if ((T_K>=0 && numtype(T_K)==0) || numtype(T_K)==2)		// passed value is OK, (NaN is VALID)
		Variable/G root:Packages:DampedHarmonic:T_K = T_K	// good value passed, set it without asking
		return T_K
	endif

	T_K = NumVarOrDefault("root:Packages:DampedHarmonic:T_K",NaN)
	Prompt T_K, "OVERRIDE Sample Temperature (K), [use NaN to disable]"
	DoPrompt "Sample Temperature", T_K
	if (V_flag)
		return T_K									// return current value, not changed
	endif
	if (T_K>=0 || numtype(T_K)==2)
		Variable/G root:Packages:DampedHarmonic:T_K = T_K	// good value entered, set it, NaN is VALID
	endif
	return T_K
End

Static Function T_SampleFromYwave(yw,Tdefault)	// get T (K) from wave yw
	Wave yw
	Variable Tdefault									// returned T if none found (this may be NaN)

	Variable T_K = NumVarOrDefault("root:Packages:DampedHarmonic:T_K",NaN)
	if (T_K>0)
		return T_K										// return OVERRIDE value
	endif

	if (!WaveExists(yw))							// wave does not exist, give up
		return Tdefault
	elseif (WaveType(yw,2)==1)					// a normal global wave, check in EPICS_PVs
		T_K = herix#TemperatureInCurrentFolder(GetWavesDataFolder(yw,1))
	endif
	if (!(T_K>0))										// look in the wave note
		T_K = NumberByKey("T_Sample",note(yw),"=")
	endif
	T_K = T_K>0 ? T_K : Tdefault
	return T_K											// Note, this can be NaN
End



Function/T CoefsTable(cw)
	Wave cw
	if (!WaveExists(cw))
		String wList = WaveListClass("DampedHarmonicCoefs","*","DIMS:1,MINROWS:4")
		wList = RemoveFromList("W_sigma",wList)
		if (ItemsInList(wList)<1)
			return ""
		elseif (ItemsInList(wList)==1)
			Wave cw = $StringFromList(0,wList)
		else
			String name
			Prompt name,"Coefficients",popup,wList
			DoPrompt "Coefficients",name
			if (V_flag)
				return ""
			endif
			Wave cw = $name
		endif
	endif
	if (!WaveExists(cw))
		return ""
	endif

	String win = StringFromList(0,WindowsWithWave(cw,2))
	if (strlen(win))
		DoWindow/F $win
	else
		String fontName="Consolas"
		Variable fontSize=12
		Edit/K=1/W=(261,44,496,184) cw.ld
		ModifyTable format(Point)=1,width(Point)=44,width(cw.l)=70,width(cw.d)=96, font=fontName, size=fontSize
		DoUpdate
		win = StringFromList(0,WinList("*",";","WIN:2"))
	endif
	return win
End


// calcualte damped harmonic amplitude from peak height
Static Function peakHeight2phononAmp(phonon,fwhm,T_K,peakHeight)
	Variable phonon			// phonon energy (can be negative)
	Variable fwhm				// fwhm, this is the actual FWHM, NOT the same as gamma (the damping factor)
	Variable T_K				// T (Kelvin)
	Variable peakHeight		// peak height

	Variable B_E, gam
	phonon = abs(phonon)
	B_E = abs(1 + 1/(exp(phonon/(kB*T_K))-1))	// Bose-Einstein occupation {1 + n(phonon)}
	gam = FWHM2gammaPhonon(phonon,fwhm)
// xxxxxxxxxxxxxxxxxxxxxxxxxx
	return peakHeight * abs(gam*phonon)/B_E		// the amp
End
//
// This funciton used to just return fwhm, but that is wrong
Static Function FWHM2gammaPhonon(phonon,fwhm) // returns damping factor gamma, NOT Lorentzian Gamma!
	Variable phonon			// phonon energy (can be negative)
	Variable fwhm				// fwhm (this is the actual FWHM, NOT that same as gamma)
	phonon = abs(phonon)
	fwhm = abs(fwhm)
	return phonon*fwhm / (phonon + fwhm/2)	// value of gamma
End


// This is the fitting function when using a resolution wave
Function DampedHarmonicResWave(coef,meV) : FitFunc
	Wave coef
	Variable meV

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ Wave resolutionWave = root:resolutionWave // measured resolution
	//CurveFitDialog/ Duplicate/FREE resolutionWave, Vsum
	//CurveFitDialog/ Vsum = resolutionWave(x) * DampedHarmonic(coef,x+meV)
	//CurveFitDialog/ f(meV) = sum(Vsum)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ meV
	//CurveFitDialog/ Coefficients 4
	//CurveFitDialog/ coef[0] = HerixEo
	//CurveFitDialog/ coef[1] = amp
	//CurveFitDialog/ coef[2] = phonon
	//CurveFitDialog/ coef[3] = gam

	// bkg is zero unless the label of the last element in coef[] is "bkg"
	Variable bkg = StringMatch(GetDimLabel(coef,0,DimSize(coef,0)-1),"bkg") ? coef[DimSize(coef,0)-1] : 0

	Wave resolutionWave = root:resolutionWave // measured resolution
	Duplicate/FREE resolutionWave, Vsum
	Vsum = resolutionWave(x) * DampedHarmonic(coef,x+meV)
	return sum(Vsum) + bkg
End


// This is the fitting function WITHOUT resolution wave
// Note, I added another abs() around (gamma*meV).
//	This goes negative for negative meV.
//
Function DampedHarmonic(coef,meV) : FitFunc
	Wave coef
	Variable meV

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ meV -= HerixEo		// shift energy by Herix offset
	//CurveFitDialog/ // Bose-Einstein occupation {1 + n(meV)}
	//CurveFitDialog/ Variable B_E = abs(1 + 1/(exp(meV/(kB*T_K))-1))
	//CurveFitDialog/ f(meV) = amp * B_E * abs(gam*meV) / ( (meV*meV-phonon*phonon)^2 + (gam*meV)^2 )
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ meV
	//CurveFitDialog/ Coefficients 4
	//CurveFitDialog/ coef[0] = HerixEo
	//CurveFitDialog/ coef[1] = amp
	//CurveFitDialog/ coef[2] = phonon
	//CurveFitDialog/ coef[3] = gam

	Variable T_K = NumVarOrDefault("root:Packages:DampedHarmonic:T_K",NaN)
	T_K = T_K > 0 ? T_K : NumberByKey("T_Sample",note(coef),"=")
	T_K = T_K > 0 ? T_K : T_K_default
	meV -= coef[0]		// shift energy by Herix offset
	Variable i, Nph=floor((DimSize(coef,0)-1)/3), total=0
	if (abs(meV/(kB*T_K)) < 1e-10)		// for small meV, use a linear expansion about zero.
		for (i=0,total=0; i<Nph; i+=1)
			total += abs(coef[3*i+1]) * abs(meV + kB*T_K) * abs(coef[3*i+3])/( (coef[3*i+2])^4 )
		endfor
	else
		Variable B_E = abs(1 + 1/(exp(meV/(kB*T_K))-1))	// Bose-Einstein occupation {1 + n(meV)}
		for (i=0,total=0; i<Nph; i+=1)
			total += abs(coef[3*i+1]) * B_E * abs(coef[3*i+3]*meV) / ( (meV*meV-coef[3*i+2]*coef[3*i+2])^2 + (coef[3*i+3]*meV)^2 )
		endfor
	endif
	return total
End

//  ============================== End of Common Functions ===============================  //
//  ======================================================================================  //




//  ======================================================================================  //
//  =================================== Start of Init ====================================  //

Function Init_DampedHarmonic()
	if (exists("root:Packages:DampedHarmonic")!=2)			// only process if folders do NOT exist
		NewDataFolder/O root:Packages								// ensure data folder exists
		NewDataFolder/O root:Packages:DampedHarmonic
		if (exists("root:Packages:DampedHarmonic:excludeHW")!=2)
			Variable/G root:Packages:DampedHarmonic:excludeHW = excludeHWdefault
		endif
		if (exists("root:Packages:DampedHarmonic:excludeHWE")!=2)
			Variable/G root:Packages:DampedHarmonic:excludeHWE = excludeHWdefaultE
		endif
		if (exists("root:Packages:DampedHarmonic:NphLast")!=2)
			Variable/G root:Packages:DampedHarmonic:NphLast = 1
		endif
	endif
	return 0
End

//  ==================================== End of Init =====================================  //
//  ======================================================================================  //
